@@ -9,7 +9,7 @@
 #                         and Technology (RIST). All rights reserved.
 # Copyright (c) 2015      IBM Corporation.  All rights reserved.
 #
-# Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+# Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
 # $COPYRIGHT$
 #
 # Additional copyrights may follow
@@ -275,6 +275,7 @@ sub mca_generate_framework_header(\$\@) {
     my (@frameworks) = @_;
     my $framework_array_output="";
     my $framework_decl_output="";
+    my $framework_name_output="";
 
     foreach my $framework (@frameworks) {
         # There is no common framework object
@@ -282,8 +283,10 @@ sub mca_generate_framework_header(\$\@) {
             my $framework_name = "pmix_${framework}_base_framework";
             $framework_array_output .= "    &$framework_name,\n";
             $framework_decl_output .= "extern pmix_mca_base_framework_t $framework_name;\n";
+            $framework_name_output .= "    \"${framework}\",\n";
         }
     }
+    $framework_name_output .= "    \"mca\",\n";
 
     my $ifdef_string = uc "pmix_FRAMEWORKS_H";
     open(FRAMEWORKS_OUT, ">src/include/pmix_frameworks.h");
@@ -296,11 +299,31 @@ sub mca_generate_framework_header(\$\@) {
 #include \"src/mca/base/pmix_mca_base_framework.h\"
 
 $framework_decl_output
-static pmix_mca_base_framework_t *pmix_frameworks[] = {
+
+PMIX_EXPORT extern pmix_mca_base_framework_t *pmix_frameworks[];
+
+PMIX_EXPORT extern char *pmix_framework_names[];
+
+#endif /* $ifdef_string */\n\n";
+    close(FRAMEWORKS_OUT);
+
+    open(FRAMEWORKS_OUT, ">src/include/pmix_frameworks.c");
+    printf FRAMEWORKS_OUT "%s", "/*
+ * This file is autogenerated by autogen.pl. Do not edit this file by hand.
+ */
+
+#include \"src/include/pmix_config.h\"
+#include \"src/include/pmix_frameworks.h\"
+
+pmix_mca_base_framework_t *pmix_frameworks[] = {
 $framework_array_output    NULL
 };
 
-#endif /* $ifdef_string */\n\n";
+char *pmix_framework_names[] = {
+$framework_name_output    NULL
+};
+
+";
     close(FRAMEWORKS_OUT);
 }
 
@@ -887,6 +910,45 @@ $step. Checking tool versions\n\n";
 
 #---------------------------------------------------------------------------
 
+++$step;
+verbose "\n$step. Checking for git submodules\n\n";
+
+# Make sure we got a submodule-full clone.  If not, abort and let a
+# human figure it out.
+if (-f ".gitmodules") {
+    open(IN, "git submodule status|")
+        || die "Can't run \"git submodule status\"";
+    while (<IN>) {
+        $_ =~ m/^(.)[0-9a-f]{40}\s+(\S+)/;
+        my $status = $1;
+        my $path   = $2;
+
+        print("=== Submodule: $path\n");
+        # Make sure the submodule is there
+        if ($status eq "-") {
+            print("    ==> ERROR: Missing
+
+The submodule \"$path\" is missing.
+
+Perhaps you forgot to \"git clone --recursive ...\", or you need to
+ \"git submodule update --init --recursive\"...?\n\n");
+            exit(1);
+        }
+
+        # See if the commit in the submodule is not the same as the
+        # commit that the git submodule thinks it should be.
+        elsif ($status eq "+") {
+            print("    ==> WARNING: Submodule hash is different than upstream.
+         If this is not intentional, you may want to run:
+         \"git submodule update --init --recursive\"\n");
+        } else {
+            print("    Local hash is what is expected by the submodule (good!)\n");
+        }
+    }
+}
+
+#---------------------------------------------------------------------------
+
 # No platform file -- write an empty list
 $m4 .= "m4_define([autogen_platform_file], [])\n\n";
 
@@ -973,7 +1035,7 @@ close(M4);
 
 # Run autoreconf
 verbose "==> Running autoreconf\n";
-my $cmd = "autoreconf -ivf --warnings=all,no-obsolete,no-override -I config";
+my $cmd = "autoreconf -ivf --warnings=all,no-obsolete,no-override -I config -I config/oac";
 safe_system($cmd);
 
 patch_autotools_output(".");
