@@ -8,7 +8,7 @@
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -20,7 +20,7 @@
 
 #include "src/include/pmix_stdint.h"
 
-#include "pmix.h"
+#include "include/pmix.h"
 
 #include "src/include/pmix_globals.h"
 
@@ -76,21 +76,14 @@ PMIX_EXPORT pmix_status_t PMIx_Spawn(const pmix_info_t job_info[], size_t ninfo,
 
     PMIX_ACQUIRE_THREAD(&pmix_global_lock);
 
-    pmix_output_verbose(2, pmix_client_globals.spawn_output, "%s pmix: spawn called",
+    pmix_output_verbose(2, pmix_client_globals.spawn_output,
+                        "%s pmix: spawn called",
                         PMIX_NAME_PRINT(&pmix_globals.myid));
 
     if (pmix_globals.init_cntr <= 0) {
         PMIX_RELEASE_THREAD(&pmix_global_lock);
         return PMIX_ERR_INIT;
     }
-
-    /* if we aren't connected, don't attempt to send */
-    if (!PMIX_PEER_IS_SERVER(pmix_globals.mypeer) &&
-        !pmix_globals.connected &&
-        !PMIX_PEER_IS_LAUNCHER(pmix_globals.mypeer)) {
-        PMIX_RELEASE_THREAD(&pmix_global_lock);
-        return PMIX_ERR_UNREACH;
-    }
     PMIX_RELEASE_THREAD(&pmix_global_lock);
 
     /* ensure the nspace (if provided) is initialized */
@@ -158,7 +151,8 @@ PMIX_EXPORT pmix_status_t PMIx_Spawn_nb(const pmix_info_t job_info[], size_t nin
         /* if I am a launcher, we default to local fork/exec */
         if (PMIX_PEER_IS_LAUNCHER(pmix_globals.mypeer)) {
             forkexec = true;
-        } else if (!PMIX_PEER_IS_SERVER(pmix_globals.mypeer)) {
+        } else if (!PMIX_PEER_IS_SERVER(pmix_globals.mypeer) ||
+                   PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
             PMIX_RELEASE_THREAD(&pmix_global_lock);
             return PMIX_ERR_UNREACH;
         }
@@ -179,7 +173,7 @@ PMIX_EXPORT pmix_status_t PMIx_Spawn_nb(const pmix_info_t job_info[], size_t nin
                     /* cycle across all the apps and set this envar */
                     for (m = 0; m < napps; m++) {
                         aptr = (pmix_app_t *) &apps[m];
-                        pmix_setenv(kv->value->data.envar.envar, kv->value->data.envar.value, true,
+                        PMIx_Setenv(kv->value->data.envar.envar, kv->value->data.envar.value, true,
                                     &aptr->env);
                     }
                 }
@@ -218,7 +212,7 @@ PMIX_EXPORT pmix_status_t PMIx_Spawn_nb(const pmix_info_t job_info[], size_t nin
             tmp = pmix_basename(aptr->cmd);
             t2 = pmix_basename(aptr->argv[0]);
             if (0 != strcmp(tmp, t2)) {
-                pmix_argv_prepend_nosize(&aptr->argv, tmp);
+                PMIx_Argv_prepend_nosize(&aptr->argv, tmp);
             }
             free(tmp);
             free(t2);
@@ -247,7 +241,7 @@ PMIX_EXPORT pmix_status_t PMIx_Spawn_nb(const pmix_info_t job_info[], size_t nin
                         return rc;
                     }
                     PMIX_LIST_FOREACH (kv, &ilist, pmix_kval_t) {
-                        pmix_setenv(kv->value->data.envar.envar, kv->value->data.envar.value, true,
+                        PMIx_Setenv(kv->value->data.envar.envar, kv->value->data.envar.value, true,
                                     &aptr->env);
                     }
                     jobenvars = true;
@@ -260,7 +254,8 @@ PMIX_EXPORT pmix_status_t PMIx_Spawn_nb(const pmix_info_t job_info[], size_t nin
 
     /* if we are a server, then process this ourselves */
     if (PMIX_PEER_IS_SERVER(pmix_globals.mypeer) &&
-        !PMIX_PEER_IS_LAUNCHER(pmix_globals.mypeer)) {
+        !PMIX_PEER_IS_LAUNCHER(pmix_globals.mypeer) &&
+        !PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
 
         if (NULL == pmix_host_server.spawn) {
             return PMIX_ERR_NOT_SUPPORTED;
