@@ -14,7 +14,7 @@
  * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2018      Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -37,6 +37,8 @@
 
 #include "src/common/pmix_attributes.h"
 #include "src/include/pmix_globals.h"
+#include "src/util/pmix_name_fns.h"
+#include "src/util/pmix_printf.h"
 
 PMIX_EXPORT const char *PMIx_Proc_state_string(pmix_proc_state_t state)
 {
@@ -162,23 +164,23 @@ PMIX_EXPORT char *PMIx_Info_directives_string(pmix_info_directives_t directives)
     char *ret;
 
     if (PMIX_INFO_QUALIFIER & directives) {
-        pmix_argv_append_nosize(&tmp, "QUALIFIER");
+        PMIx_Argv_append_nosize(&tmp, "QUALIFIER");
     } else {
         if (PMIX_INFO_REQD & directives) {
-            pmix_argv_append_nosize(&tmp, "REQUIRED");
+            PMIx_Argv_append_nosize(&tmp, "REQUIRED");
         } else {
-            pmix_argv_append_nosize(&tmp, "OPTIONAL");
+            PMIx_Argv_append_nosize(&tmp, "OPTIONAL");
         }
         if (PMIX_INFO_REQD_PROCESSED & directives) {
-            pmix_argv_append_nosize(&tmp, "PROCESSED");
+            PMIx_Argv_append_nosize(&tmp, "PROCESSED");
         }
         if (PMIX_INFO_ARRAY_END & directives) {
-            pmix_argv_append_nosize(&tmp, "END");
+            PMIx_Argv_append_nosize(&tmp, "END");
         }
     }
     if (NULL != tmp) {
-        ret = pmix_argv_join(tmp, ':');
-        pmix_argv_free(tmp);
+        ret = PMIx_Argv_join(tmp, ':');
+        PMIx_Argv_free(tmp);
     } else {
         ret = strdup("UNSPECIFIED");
     }
@@ -266,6 +268,8 @@ PMIX_EXPORT const char *pmix_command_string(pmix_cmd_t cmd)
         return "FABRIC UPDATE";
     case PMIX_COMPUTE_DEVICE_DISTANCES_CMD:
         return "COMPUTE DEVICE DIST";
+    case PMIX_REFRESH_CACHE:
+        return "REFRESH CACHE";
     default:
         return "UNKNOWN";
     }
@@ -392,3 +396,80 @@ const char* PMIx_Value_comparison_string(pmix_value_cmp_t cmp)
         return "UNKNOWN VALUE";
     }
 }
+
+char* PMIx_App_string(const pmix_app_t *app)
+{
+    char **ans = NULL;
+    char *tmp, *str;
+    size_t n;
+
+    /* add the command */
+    pmix_asprintf(&tmp, "CMD: %s", app->cmd);
+    PMIx_Argv_append_nosize(&ans, tmp);
+    free(tmp);
+
+    /* add the argv */
+    PMIx_Argv_append_nosize(&ans, "    ARGV:");
+    if (NULL == app->argv) {
+        PMIx_Argv_append_nosize(&ans, "        NONE");
+    } else {
+        for (n=0; NULL != app->argv[n]; n++) {
+            pmix_asprintf(&tmp, "        ARGV[%d]: %s", (int)n, app->argv[n]);
+            PMIx_Argv_append_nosize(&ans, tmp);
+            free(tmp);
+        }
+    }
+
+    /* add the env */
+    PMIx_Argv_append_nosize(&ans, "    ENV:");
+    if (NULL == app->env) {
+        PMIx_Argv_append_nosize(&ans, "        NONE");
+    } else {
+        for (n=0; NULL != app->env[n]; n++) {
+            pmix_asprintf(&tmp, "        ENV[%d]: %s", (int)n, app->env[n]);
+            PMIx_Argv_append_nosize(&ans, tmp);
+            free(tmp);
+        }
+    }
+
+    /* add the cwd */
+    if (NULL == app->cwd) {
+        PMIx_Argv_append_nosize(&ans, "    CWD: NULL");
+    } else {
+        pmix_asprintf(&tmp, "    CWD: %s", app->cwd);
+        PMIx_Argv_append_nosize(&ans, tmp);
+        free(tmp);
+    }
+
+    /* add the maxprocs */
+    pmix_asprintf(&tmp, "    MAXPROCS: %d", app->maxprocs);
+    PMIx_Argv_append_nosize(&ans, tmp);
+    free(tmp);
+
+    /* add any info */
+    if (NULL == app->info) {
+        PMIx_Argv_append_nosize(&ans, "    INFO: NONE");
+    } else {
+        PMIx_Argv_append_nosize(&ans, "    INFO:");
+        for (n=0; n < app->ninfo; n++) {
+            str = PMIx_Info_string(&app->info[n]);
+            pmix_asprintf(&tmp, "        INFO[%d]: %s", (int)n, str);
+            PMIx_Argv_append_nosize(&ans, tmp);
+            free(tmp);
+            free(str);
+        }
+    }
+
+    tmp = PMIx_Argv_join(ans, '\n');
+    PMIx_Argv_free(ans);
+    return tmp;
+}
+
+char* PMIx_Proc_string(const pmix_proc_t *proc)
+{
+    char *tmp, *result;
+
+    tmp = pmix_util_print_name_args(proc);
+    result = strdup(tmp);
+    return result;
+}
