@@ -5,7 +5,7 @@
  *                         All rights reserved.
  * Copyright (c) 2018      Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
  * Copyright (c) 2022      Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
@@ -109,8 +109,7 @@ pmix_status_t pmix_hwloc_register(void)
     vmhole = "biggest";
     (void) pmix_mca_base_var_register("pmix", "pmix", "hwloc", "hole_kind",
                                       "Kind of VM hole to identify - none, begin, biggest, libs, heap, stack (default=biggest)",
-                                      PMIX_MCA_BASE_VAR_TYPE_STRING,
-                                      &vmhole);
+                                      PMIX_MCA_BASE_VAR_TYPE_STRING, &vmhole);
     if (0 == strcasecmp(vmhole, "none")) {
         hole_kind = VMEM_HOLE_NONE;
     } else if (0 == strcasecmp(vmhole, "begin")) {
@@ -130,8 +129,7 @@ pmix_status_t pmix_hwloc_register(void)
 
     (void) pmix_mca_base_var_register("pmix", "pmix", "hwloc", "topo_file",
                                       "Topology file to use instead of discovering it (mostly for testing purposes)",
-                                      PMIX_MCA_BASE_VAR_TYPE_STRING,
-                                      &topo_file);
+                                      PMIX_MCA_BASE_VAR_TYPE_STRING, &topo_file);
 
     (void) pmix_mca_base_var_register("pmix", "pmix", "hwloc", "test_cpuset",
                                       "Cpuset for testing purposes",
@@ -321,33 +319,40 @@ tryxml:
     PMIX_GDS_FETCH_KV(rc, pmix_client_globals.myserver, &cb);
     if (PMIX_SUCCESS == rc) {
         file = popstr(&cb);
-        rc = load_xml(file);
-        free(file);
+        if (NULL == file) {
+            rc = PMIX_ERR_NOT_FOUND;
+        } else {
+            rc = load_xml(file);
+            free(file);
+        }
         cb.key = NULL;
         PMIX_DESTRUCT(&cb);
-        if (PMIX_SUCCESS == rc) {
-            pmix_output_verbose(2, pmix_hwloc_output,
-                                "%s:%s v2 xml adopted", __FILE__, __func__);
-            /* record locally in case someone does a PMIx_Get to retrieve it */
-            kv.key = PMIX_TOPOLOGY2;
-            kv.value = &val;
-            val.type = PMIX_TOPO;
-            val.data.topo = &pmix_globals.topology;
-            PMIX_GDS_STORE_KV(rc, pmix_globals.mypeer, &pmix_globals.myid, PMIX_INTERNAL, &kv);
-            pmix_output_verbose(2, pmix_hwloc_output, "%s:%s stored",
-                                __FILE__, __func__);
-            if (PMIX_SUCCESS != rc) {
-                PMIX_ERROR_LOG(rc);
-            }
-            if (share) {
-                goto sharetopo;
-            }
+        if (PMIX_SUCCESS != rc) {
+            goto tryv1;
+        }
+
+        pmix_output_verbose(2, pmix_hwloc_output,
+                            "%s:%s v2 xml adopted", __FILE__, __func__);
+        /* record locally in case someone does a PMIx_Get to retrieve it */
+        kv.key = PMIX_TOPOLOGY2;
+        kv.value = &val;
+        val.type = PMIX_TOPO;
+        val.data.topo = &pmix_globals.topology;
+        PMIX_GDS_STORE_KV(rc, pmix_globals.mypeer, &pmix_globals.myid, PMIX_INTERNAL, &kv);
+        pmix_output_verbose(2, pmix_hwloc_output, "%s:%s stored",
+                            __FILE__, __func__);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
+        }
+        if (share) {
+            goto sharetopo;
         }
         return rc;
     }
 
 #endif
 
+tryv1:
     /* try to get the v1 XML string */
     pmix_output_verbose(2, pmix_hwloc_output,
                         "%s:%s checking v1 xml",
@@ -359,35 +364,38 @@ tryxml:
     PMIX_GDS_FETCH_KV(rc, pmix_client_globals.myserver, &cb);
     if (PMIX_SUCCESS == rc) {
         file = popstr(&cb);
-        rc = load_xml(file);
-        free(file);
+        if (NULL == file) {
+            rc = PMIX_ERR_NOT_FOUND;
+        } else {
+            rc = load_xml(file);
+            free(file);
+        }
         cb.key = NULL;
         PMIX_DESTRUCT(&cb);
-        if (PMIX_SUCCESS == rc) {
-            pmix_output_verbose(2, pmix_hwloc_output,
-                                "%s:%s v1 xml adopted", __FILE__, __func__);
+        if (PMIX_SUCCESS != rc) {
+            goto tryself;
+        }
+        pmix_output_verbose(2, pmix_hwloc_output,
+                            "%s:%s v1 xml adopted", __FILE__, __func__);
 
-            /* record locally in case someone does a PMIx_Get to retrieve it */
-            kv.key = PMIX_TOPOLOGY2;
-            kv.value = &val;
-            val.type = PMIX_TOPO;
-            val.data.topo = &pmix_globals.topology;
-            PMIX_GDS_STORE_KV(rc, pmix_globals.mypeer, &pmix_globals.myid, PMIX_INTERNAL, &kv);
-            pmix_output_verbose(2, pmix_hwloc_output, "%s:%s stored",
-                                __FILE__, __func__);
-            if (PMIX_SUCCESS != rc) {
-                PMIX_ERROR_LOG(rc);
-            }
-            if (share) {
-                goto sharetopo;
-            }
+        /* record locally in case someone does a PMIx_Get to retrieve it */
+        kv.key = PMIX_TOPOLOGY2;
+        kv.value = &val;
+        val.type = PMIX_TOPO;
+        val.data.topo = &pmix_globals.topology;
+        PMIX_GDS_STORE_KV(rc, pmix_globals.mypeer, &pmix_globals.myid, PMIX_INTERNAL, &kv);
+        pmix_output_verbose(2, pmix_hwloc_output, "%s:%s stored",
+                            __FILE__, __func__);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
+        }
+        if (share) {
+            goto sharetopo;
         }
         return rc;
     }
 
-#if HWLOC_API_VERSION >= 0x20000
 tryself:
-#endif
     /* did they give us one to use? */
     if (NULL != topo_file) {
         pmix_output_verbose(2, pmix_hwloc_output,
@@ -440,6 +448,7 @@ tryself:
                             "%s:%s discovery complete - source %s", __FILE__, __func__,
                             pmix_globals.topology.source);
     }
+
     /* record locally in case someone does a PMIx_Get to retrieve it */
     kv.key = PMIX_TOPOLOGY2;
     kv.value = &val;
@@ -777,7 +786,7 @@ pmix_status_t pmix_hwloc_generate_locality_string(const pmix_cpuset_t *cpuset, c
         /* get the object type at this depth */
         type = hwloc_get_depth_type(pmix_globals.topology.topology, d);
         /* if it isn't one of interest, then ignore it */
-        if (HWLOC_OBJ_NODE != type && HWLOC_OBJ_PACKAGE != type &&
+        if (HWLOC_OBJ_NUMANODE != type && HWLOC_OBJ_PACKAGE != type &&
 #if HWLOC_API_VERSION < 0x20000
             HWLOC_OBJ_CACHE != type &&
 #else
@@ -796,7 +805,7 @@ pmix_status_t pmix_hwloc_generate_locality_string(const pmix_cpuset_t *cpuset, c
         if (!hwloc_bitmap_iszero(result)) {
             hwloc_bitmap_list_asprintf(&tmp, result);
             switch (type) {
-                case HWLOC_OBJ_NODE:
+                case HWLOC_OBJ_NUMANODE:
                     pmix_asprintf(&t2, "%sNM%s:", (NULL == locality) ? "" : locality, tmp);
                     if (NULL != locality) {
                         free(locality);
@@ -932,8 +941,8 @@ pmix_status_t pmix_hwloc_get_relative_locality(const char *locality1,
     /* start with what we know - they share a node */
     locality = PMIX_LOCALITY_SHARE_NODE;
 
-    set1 = pmix_argv_split(loc1, ':');
-    set2 = pmix_argv_split(loc2, ':');
+    set1 = PMIx_Argv_split(loc1, ':');
+    set2 = PMIx_Argv_split(loc2, ':');
     bit1 = hwloc_bitmap_alloc();
     bit2 = hwloc_bitmap_alloc();
 
@@ -973,8 +982,8 @@ pmix_status_t pmix_hwloc_get_relative_locality(const char *locality1,
             }
         }
     }
-    pmix_argv_free(set1);
-    pmix_argv_free(set2);
+    PMIx_Argv_free(set1);
+    PMIx_Argv_free(set2);
     hwloc_bitmap_free(bit1);
     hwloc_bitmap_free(bit2);
     *loc = locality;
@@ -1005,7 +1014,7 @@ pmix_status_t pmix_hwloc_get_cpuset(pmix_cpuset_t *cpuset, pmix_bind_envelope_t
     }
     if (0 != rc) {
         hwloc_bitmap_free(cpuset->bitmap);
-        return PMIX_ERR_TAKE_NEXT_OPTION;
+        return PMIX_ERR_NOT_FOUND;
     }
     if (NULL == cpuset->source) {
         cpuset->source = strdup("hwloc");
@@ -1139,7 +1148,7 @@ pmix_status_t pmix_hwloc_compute_distances(pmix_topology_t *topo, pmix_cpuset_t
             if (PMIX_CHECK_KEY(&info[n], PMIX_DEVICE_TYPE)) {
                 type |= info[n].value.data.devtype;
             } else if (PMIX_CHECK_KEY(&info[n], PMIX_DEVICE_ID)) {
-                pmix_argv_append_nosize(&devids, info[n].value.data.string);
+                PMIx_Argv_append_nosize(&devids, info[n].value.data.string);
             }
         }
     }
