@@ -15,7 +15,8 @@
  *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2019      Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2023      Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -66,9 +67,25 @@ BEGIN_C_DECLS
     "pmix.pnet.setapp" // (pmix_byte_object_t) blob containing info to be given to
                        //      pnet framework on remote nodes
 
+// define some bit handling macros
+#define PMIX_SET_BIT(a, f) \
+    (a) |= (f)
+
+#define PMIX_UNSET_BIT(a, f) \
+    (a) &= ~(f)
+
+#define PMIX_CHECK_BIT_IS_SET(a, f) \
+    ((a) & (f))
+
+#define PMIX_CHECK_BIT_NOT_SET(a, f) \
+    !PMIX_CHECK_BIT_IS_SET(a, f)
+
 #define PMIX_INFO_OP_COMPLETE       0x80000000
-#define PMIX_INFO_OP_COMPLETED(m)   ((pmix_info_t *) (m))->flags |= PMIX_INFO_OP_COMPLETE
-#define PMIX_INFO_OP_IS_COMPLETE(m) ((m)->flags & PMIX_INFO_OP_COMPLETE)
+#define PMIX_INFO_OP_COMPLETED(m) \
+      PMIX_SET_BIT((m)->flags, PMIX_INFO_OP_COMPLETE)
+#define PMIX_INFO_OP_IS_COMPLETE(m) \
+      PMIX_CHECK_BIT_IS_SET((m)->flags, PMIX_INFO_OP_COMPLETE)
+
 
 /* define an internal-only object for creating
  * lists of names */
@@ -88,6 +105,14 @@ typedef struct {
     char **description;
 } pmix_regattr_input_t;
 
+/* define a struct for holding entries in the
+ * dictionary of event strings */
+typedef struct {
+    uint32_t index;
+    char *name;
+    int32_t code;
+} pmix_event_string_t;
+
 /* define a struct for storing data in memory */
 typedef struct {
     uint32_t index;
@@ -113,23 +138,28 @@ typedef struct {
     uint32_t qualindex;
     pmix_value_t *value;
 } pmix_dstor_t;
+
+PMIX_EXPORT pmix_dstor_t *
+pmix_dstor_new_tma(
+    uint32_t index,
+    pmix_tma_t *tma
+);
+
+PMIX_EXPORT void
+pmix_dstor_release_tma(
+    pmix_dstor_t *d,
+    pmix_tma_t *tma
+);
+
 #define PMIX_DSTOR_NEW(d, k)                                \
 do {                                                        \
-    (d) = (pmix_dstor_t*)pmix_malloc(sizeof(pmix_dstor_t)); \
-    if (NULL != (d)) {                                      \
-        (d)->index = k;                                     \
-        (d)->qualindex = UINT32_MAX;                        \
-        (d)->value = NULL;                                  \
-    }                                                       \
-} while(0)
-#define PMIX_DSTOR_RELEASE(d)           \
-do {                                    \
-    if (NULL != (d)->value) {           \
-        PMIX_VALUE_RELEASE((d)->value); \
-    }                                   \
-    free(d);                            \
+    (d) = pmix_dstor_new_tma((k), NULL);                    \
 } while(0)
 
+#define PMIX_DSTOR_RELEASE(d)           \
+do {                                    \
+    pmix_dstor_release_tma((d), NULL);  \
+} while(0)
 
 /* define a struct for passing topology objects */
 typedef struct {
@@ -177,6 +207,7 @@ typedef uint8_t pmix_cmd_t;
 #define PMIX_FABRIC_REGISTER_CMD          30
 #define PMIX_FABRIC_UPDATE_CMD            31
 #define PMIX_COMPUTE_DEVICE_DISTANCES_CMD 32
+#define PMIX_REFRESH_CACHE                33
 
 /* provide a "pretty-print" function for cmds */
 const char *pmix_command_string(pmix_cmd_t cmd);
@@ -269,7 +300,6 @@ typedef struct {
     .raw = false                    \
 }
 
-
 /* objects used by servers for tracking active nspaces */
 typedef struct {
     pmix_list_item_t super;
@@ -414,6 +444,7 @@ typedef struct {
     pmix_object_t super;
     pmix_event_t ev;
     pmix_lock_t lock;
+    bool host_called;
     pmix_status_t status;
     pmix_query_t *queries;
     size_t nqueries;
@@ -437,6 +468,14 @@ typedef struct {
 } pmix_query_caddy_t;
 PMIX_CLASS_DECLARATION(pmix_query_caddy_t);
 
+typedef struct {
+    pmix_list_item_t super;
+    char *grpid;
+    pmix_proc_t *members;
+    size_t nmbrs;
+} pmix_group_t;
+PMIX_CLASS_DECLARATION(pmix_group_t);
+
 typedef struct {
     pmix_list_item_t super;
     pmix_proc_t proc;
@@ -455,20 +494,19 @@ typedef struct {
     char *id;         // string identifier for the collective
     pmix_cmd_t type;
     pmix_proc_t pname;
-    bool hybrid;        // true if participating procs are from more than one nspace
-    pmix_proc_t *pcs;   // copy of the original array of participants
-    size_t npcs;        // number of procs in the array
-    pmix_list_t nslist; // unique nspace list of participants
-    pmix_lock_t lock;   // flag for waiting for completion
-    bool def_complete;  // all local procs have been registered and the trk definition is complete
-    pmix_list_t
-        local_cbs;      // list of pmix_server_caddy_t for sending result to the local participants
-                        //    Note: there may be multiple entries for a given proc if that proc
-                        //    has fork/exec'd clones that are also participating
-    uint32_t nlocal;    // number of local participants
-    uint32_t local_cnt; // number of local participants who have contributed
-    pmix_info_t *info;  // array of info structs
-    size_t ninfo;       // number of info structs in array
+    bool hybrid;            // true if participating procs are from more than one nspace
+    pmix_proc_t *pcs;       // copy of the original array of participants
+    size_t npcs;            // number of procs in the array
+    pmix_list_t nslist;     // unique nspace list of participants
+    pmix_lock_t lock;       // flag for waiting for completion
+    bool def_complete;      // all local procs have been registered and the trk definition is complete
+    pmix_list_t local_cbs;  // list of pmix_server_caddy_t for sending result to the local participants
+                            //    Note: there may be multiple entries for a given proc if that proc
+                            //    has fork/exec'd clones that are also participating
+    uint32_t nlocal;        // number of local participants
+    uint32_t local_cnt;     // number of local participants who have contributed
+    pmix_info_t *info;      // array of info structs
+    size_t ninfo;           // number of info structs in array
     pmix_list_t grpinfo;    // list of group info to be distributed
     pmix_collect_t collect_type; // whether or not data is to be returned at completion
     pmix_modex_cbfunc_t modexcbfunc;
@@ -502,6 +540,7 @@ typedef struct {
     pmix_status_t status;
     pmix_status_t *codes;
     size_t ncodes;
+    uint32_t sessionid;
     pmix_name_t pname;
     pmix_proc_t *proc;
     pmix_peer_t *peer;
@@ -537,10 +576,20 @@ typedef struct {
     bool pntrval;
     bool stval;
     bool optional;
+    bool immediate;
     bool add_immediate;
-    bool qualified_value;
     bool refresh_cache;
     pmix_scope_t scope;
+    bool sessioninfo;
+    bool sessiondirective;
+    uint32_t sessionid;
+    bool nodeinfo;
+    bool nodedirective;
+    char *hostname;
+    uint32_t nodeid;
+    bool appinfo;
+    bool appdirective;
+    uint32_t appnum;
 } pmix_get_logic_t;
 PMIX_CLASS_DECLARATION(pmix_get_logic_t);
 
@@ -664,9 +713,10 @@ typedef struct {
     uint32_t appnum;     // my appnum
     pid_t pid;           // my local pid
     uint32_t nodeid;     // my nodeid, if given
-    uint32_t sessionid;
+    uint32_t sessionid;  // my sessionid, if given
     int pindex;
     pmix_event_base_t *evbase;
+    pmix_event_base_t *evauxbase;
     int debug_output;
     pmix_events_t events; // my event handler registrations.
     bool connected;
@@ -770,6 +820,23 @@ static inline bool pmix_check_session_info(const char *key)
     return false;
 }
 
+static inline bool pmix_check_special_key(const char *key)
+{
+    char *keys[] = {
+        PMIX_GROUP_CONTEXT_ID,
+        PMIX_GROUP_LOCAL_CID,
+        NULL
+    };
+    size_t n;
+
+    for (n = 0; NULL != keys[n]; n++) {
+        if (0 == strncmp(key, keys[n], PMIX_MAX_KEYLEN)) {
+            return true;
+        }
+    }
+    return false;
+}
+
 #if PMIX_PICKY_COMPILERS
 #define PMIX_HIDE_UNUSED_PARAMS(...)                \
     do {                                            \
