@@ -48,12 +48,6 @@ do {                                \
     }                               \
 } while(0)
 
-PMIX_EXPORT pmix_value_cmp_t PMIx_Value_compare(pmix_value_t *v1,
-                                                pmix_value_t *v2)
-{
-    return pmix_bfrops_base_value_cmp(v1, v2);
-}
-
 static pmix_value_cmp_t cmp_string(char *s1, char *s2)
 {
     int ret;
@@ -896,8 +890,8 @@ static pmix_value_cmp_t cmp_regattr(pmix_regattr_t *r1,
         return PMIX_VALUE2_GREATER;
     }
     /* both are non-NULL */
-    c1 = pmix_argv_count(r1->description);
-    c2 = pmix_argv_count(r2->description);
+    c1 = PMIx_Argv_count(r1->description);
+    c2 = PMIx_Argv_count(r2->description);
     if (c1 > c2) {
         return PMIX_VALUE1_GREATER;
     } else if (c2 > c1) {
