@@ -115,9 +115,8 @@ pmix_status_t pmix_gds_base_store_modex(struct pmix_namespace_t *nspace, pmix_bu
     PMIX_BFROPS_UNPACK(rc, pmix_globals.mypeer, buff, &bo, &cnt, PMIX_BYTE_OBJECT);
 
     /* If the collect flag is set, we should have some data for unpacking */
-    if ((PMIX_COLLECT_YES == trk->collect_type) &&
-        (PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER == rc)) {
-        PMIX_ERROR_LOG(rc);
+    if ((PMIX_COLLECT_YES == trk->collect_type)
+        && (PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER == rc)) {
         goto exit;
     }
 
@@ -130,7 +129,6 @@ pmix_status_t pmix_gds_base_store_modex(struct pmix_namespace_t *nspace, pmix_bu
         if (PMIX_ERR_UNPACK_READ_PAST_END_OF_BUFFER == rc) {
             /* no data was returned, so we are done with this blob */
             PMIX_DESTRUCT(&bkt);
-            PMIX_ERROR_LOG(rc);
             break;
         }
         if (PMIX_SUCCESS != rc) {
@@ -180,7 +178,7 @@ pmix_status_t pmix_gds_base_store_modex(struct pmix_namespace_t *nspace, pmix_bu
                 PMIX_DESTRUCT(&bkt);
                 goto exit;
             }
-            if (pmix_argv_count(kmap) != (int) kmap_size) {
+            if (PMIx_Argv_count(kmap) != (int) kmap_size) {
                 rc = PMIX_ERR_UNPACK_FAILURE;
                 PMIX_ERROR_LOG(rc);
                 PMIX_DESTRUCT(&bkt);
@@ -211,7 +209,7 @@ pmix_status_t pmix_gds_base_store_modex(struct pmix_namespace_t *nspace, pmix_bu
                 break;
             }
             found = false;
-            /* calculate proc form the relative rank */
+            /* calculate proc from the relative rank */
             if (pmix_list_get_size(&trk->nslist) == 1) {
                 found = true;
                 nm = (pmix_nspace_caddy_t *) pmix_list_get_first(&trk->nslist);
@@ -268,7 +266,7 @@ pmix_status_t pmix_gds_base_store_modex(struct pmix_namespace_t *nspace, pmix_bu
         PMIX_ERROR_LOG(rc);
     }
 exit:
-    pmix_argv_free(kmap);
+    PMIx_Argv_free(kmap);
     return rc;
 }
 
