@@ -10,6 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2019      Intel, Inc.  All rights reserved.
+ * Copyright (c) 2022      Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -35,7 +36,7 @@ BEGIN_C_DECLS
  * PFEXEC Linux module
  */
 PMIX_EXPORT extern pmix_pfexec_base_module_t pmix_pfexec_linux_module;
-PMIX_EXPORT extern pmix_pfexec_base_component_t mca_pfexec_linux_component;
+PMIX_EXPORT extern pmix_pfexec_base_component_t pmix_mca_pfexec_linux_component;
 
 END_C_DECLS
 
