@@ -47,27 +47,20 @@ static pmix_status_t component_register(void);
  */
 pmix_pnet_nvd_component_t pmix_mca_pnet_nvd_component = {
     .super = {
-        .base = {
-            PMIX_PNET_BASE_VERSION_1_0_0,
+        PMIX_PNET_BASE_VERSION_1_0_0,
 
-            /* Component name and version */
-            .pmix_mca_component_name = "nvd",
-            PMIX_MCA_BASE_MAKE_VERSION(component,
-                                       PMIX_MAJOR_VERSION,
-                                       PMIX_MINOR_VERSION,
-                                       PMIX_RELEASE_VERSION),
+        /* Component name and version */
+        .pmix_mca_component_name = "nvd",
+        PMIX_MCA_BASE_MAKE_VERSION(component,
+                                   PMIX_MAJOR_VERSION,
+                                   PMIX_MINOR_VERSION,
+                                   PMIX_RELEASE_VERSION),
 
-            /* Component open and close functions */
-            .pmix_mca_open_component = component_open,
-            .pmix_mca_close_component = component_close,
-            .pmix_mca_register_component_params = component_register,
-            .pmix_mca_query_component = component_query,
-        },
-        .data = {
-            /* The component is checkpoint ready */
-            PMIX_MCA_BASE_METADATA_PARAM_CHECKPOINT,
-            .reserved = {0}
-        }
+        /* Component open and close functions */
+        .pmix_mca_open_component = component_open,
+        .pmix_mca_close_component = component_close,
+        .pmix_mca_register_component_params = component_register,
+        .pmix_mca_query_component = component_query,
     },
     .include = NULL,
     .exclude = NULL
@@ -75,26 +68,26 @@ pmix_pnet_nvd_component_t pmix_mca_pnet_nvd_component = {
 
 static pmix_status_t component_register(void)
 {
-    pmix_mca_base_component_t *component = &pmix_mca_pnet_nvd_component.super.base;
+    pmix_mca_base_component_t *component = &pmix_mca_pnet_nvd_component.super;
 
     pmix_mca_pnet_nvd_component.incparms = "UCX_*,HCOLL_*,UCC_*,SHARP_*,NCCL_*";
-    (void) pmix_mca_base_component_var_register(component,
-                                                "include_envars",
-                                                "Comma-delimited list of envars to harvest (\'*\' and \'?\' supported)",
-                                                PMIX_MCA_BASE_VAR_TYPE_STRING,
-                                                &pmix_mca_pnet_nvd_component.incparms);
+    (void) pmix_mca_base_component_var_register(
+        component, "include_envars",
+        "Comma-delimited list of envars to harvest (\'*\' and \'?\' supported)",
+        PMIX_MCA_BASE_VAR_TYPE_STRING,
+        &pmix_mca_pnet_nvd_component.incparms);
     if (NULL != pmix_mca_pnet_nvd_component.incparms) {
-        pmix_mca_pnet_nvd_component.include = pmix_argv_split(pmix_mca_pnet_nvd_component.incparms, ',');
+        pmix_mca_pnet_nvd_component.include = PMIx_Argv_split(pmix_mca_pnet_nvd_component.incparms, ',');
     }
 
     pmix_mca_pnet_nvd_component.excparms = NULL;
-    (void) pmix_mca_base_component_var_register(component,
-                                                "exclude_envars",
-                                                "Comma-delimited list of envars to exclude (\'*\' and \'?\' supported)",
-                                                PMIX_MCA_BASE_VAR_TYPE_STRING,
-                                                &pmix_mca_pnet_nvd_component.excparms);
+    (void) pmix_mca_base_component_var_register(
+        component, "exclude_envars",
+        "Comma-delimited list of envars to exclude (\'*\' and \'?\' supported)",
+        PMIX_MCA_BASE_VAR_TYPE_STRING,
+        &pmix_mca_pnet_nvd_component.excparms);
     if (NULL != pmix_mca_pnet_nvd_component.excparms) {
-        pmix_mca_pnet_nvd_component.exclude = pmix_argv_split(pmix_mca_pnet_nvd_component.excparms, ',');
+        pmix_mca_pnet_nvd_component.exclude = PMIx_Argv_split(pmix_mca_pnet_nvd_component.excparms, ',');
     }
 
     return PMIX_SUCCESS;
