@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2018-2020 Intel, Inc.  All rights reserved.
  *
- * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -31,7 +31,7 @@ typedef struct {
 } pmix_pnet_tcp_component_t;
 
 /* the component must be visible data for the linker to find it */
-PMIX_EXPORT extern pmix_pnet_tcp_component_t mca_pnet_tcp_component;
+PMIX_EXPORT extern pmix_pnet_tcp_component_t pmix_mca_pnet_tcp_component;
 extern pmix_pnet_module_t pmix_tcp_module;
 
 END_C_DECLS
