@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2015-2020 Intel, Inc.  All rights reserved.
  *
- * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,7 +29,7 @@ typedef struct {
 } pmix_pnet_usnic_component_t;
 
 /* the component must be visible data for the linker to find it */
-PMIX_EXPORT extern pmix_pnet_usnic_component_t mca_pnet_usnic_component;
+PMIX_EXPORT extern pmix_pnet_usnic_component_t pmix_mca_pnet_usnic_component;
 extern pmix_pnet_module_t pmix_pnet_usnic_module;
 
 /* define a key for any blob we need to send in a launch msg */
