@@ -1,7 +1,7 @@
 /* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2018-2020 Intel, Inc.  All rights reserved.
- * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -16,7 +16,7 @@
  */
 
 #include "src/include/pmix_config.h"
-#include "include/pmix_common.h"
+#include "pmix_common.h"
 
 #include "prm_default.h"
 #include "src/mca/prm/prm.h"
@@ -27,24 +27,18 @@ static pmix_status_t component_query(pmix_mca_base_module_t **module, int *prior
  * Instantiate the public struct with all of our public information
  * and pointers to our public functions in it
  */
-pmix_prm_base_component_t mca_prm_default_component = {
-    .base = {
-        PMIX_PRM_BASE_VERSION_1_0_0,
+pmix_prm_base_component_t pmix_mca_prm_default_component = {
+    PMIX_PRM_BASE_VERSION_1_0_0,
 
-        /* Component name and version */
-        .pmix_mca_component_name = "default",
-        PMIX_MCA_BASE_MAKE_VERSION(component,
-                                   PMIX_MAJOR_VERSION,
-                                   PMIX_MINOR_VERSION,
-                                   PMIX_RELEASE_VERSION),
+    /* Component name and version */
+    .pmix_mca_component_name = "default",
+    PMIX_MCA_BASE_MAKE_VERSION(component,
+                               PMIX_MAJOR_VERSION,
+                               PMIX_MINOR_VERSION,
+                               PMIX_RELEASE_VERSION),
 
-        /* Component open and close functions */
-        .pmix_mca_query_component = component_query,
-    },
-    .data = {
-        /* The component is checkpoint ready */
-        PMIX_MCA_BASE_METADATA_PARAM_CHECKPOINT
-    }
+    /* Component open and close functions */
+    .pmix_mca_query_component = component_query,
 };
 
 static pmix_status_t component_query(pmix_mca_base_module_t **module, int *priority)
