@@ -35,11 +35,11 @@ char *pmix_psec_base_get_available_modules(void)
     }
 
     PMIX_LIST_FOREACH (active, &pmix_psec_globals.actives, pmix_psec_base_active_module_t) {
-        pmix_argv_append_nosize(&tmp, active->component->base.pmix_mca_component_name);
+        PMIx_Argv_append_nosize(&tmp, active->component->base.pmix_mca_component_name);
     }
     if (NULL != tmp) {
-        reply = pmix_argv_join(tmp, ',');
-        pmix_argv_free(tmp);
+        reply = PMIx_Argv_join(tmp, ',');
+        PMIx_Argv_free(tmp);
     }
     return reply;
 }
@@ -56,7 +56,7 @@ pmix_psec_module_t *pmix_psec_base_assign_module(const char *options)
     }
 
     if (NULL != options) {
-        tmp = pmix_argv_split(options, ',');
+        tmp = PMIx_Argv_split(options, ',');
     }
 
     PMIX_LIST_FOREACH (active, &pmix_psec_globals.actives, pmix_psec_base_active_module_t) {
@@ -68,7 +68,7 @@ pmix_psec_module_t *pmix_psec_base_assign_module(const char *options)
             for (i = 0; NULL != tmp[i]; i++) {
                 if (0 == strcmp(tmp[i], active->component->base.pmix_mca_component_name)) {
                     if (NULL != (mod = active->component->assign_module())) {
-                        pmix_argv_free(tmp);
+                        PMIx_Argv_free(tmp);
                         return mod;
                     }
                 }
@@ -78,7 +78,7 @@ pmix_psec_module_t *pmix_psec_base_assign_module(const char *options)
 
     /* we only get here if nothing was found */
     if (NULL != tmp) {
-        pmix_argv_free(tmp);
+        PMIx_Argv_free(tmp);
     }
     return NULL;
 }
