@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2015-2020 Intel, Inc.  All rights reserved.
  *
- * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -23,7 +23,7 @@ typedef struct {
 } pmix_pstrg_vfs_component_t;
 
 /* the component must be visible data for the linker to find it */
-PMIX_EXPORT extern pmix_pstrg_vfs_component_t mca_pstrg_vfs_component;
+PMIX_EXPORT extern pmix_pstrg_vfs_component_t pmix_mca_pstrg_vfs_component;
 extern pmix_pstrg_base_module_t pmix_pstrg_vfs_module;
 
 END_C_DECLS
