@@ -220,8 +220,8 @@ pmix_status_t pmix_ptl_base_setup_fork(const pmix_proc_t *proc, char ***env)
 {
     PMIX_HIDE_UNUSED_PARAMS(proc);
 
-    pmix_setenv("PMIX_SERVER_TMPDIR", pmix_ptl_base.session_tmpdir, true, env);
-    pmix_setenv("PMIX_SYSTEM_TMPDIR", pmix_ptl_base.system_tmpdir, true, env);
+    PMIx_Setenv("PMIX_SERVER_TMPDIR", pmix_ptl_base.session_tmpdir, true, env);
+    PMIx_Setenv("PMIX_SYSTEM_TMPDIR", pmix_ptl_base.system_tmpdir, true, env);
 
     return PMIX_SUCCESS;
 }
@@ -232,10 +232,10 @@ pmix_status_t pmix_ptl_base_parse_uri(const char *evar, char **nspace, pmix_rank
     char **uri;
     char *p;
 
-    uri = pmix_argv_split(evar, ';');
-    if (2 != pmix_argv_count(uri)) {
+    uri = PMIx_Argv_split(evar, ';');
+    if (2 != PMIx_Argv_count(uri)) {
         PMIX_ERROR_LOG(PMIX_ERR_BAD_PARAM);
-        pmix_argv_free(uri);
+        PMIx_Argv_free(uri);
         return PMIX_ERR_NOT_SUPPORTED;
     }
 
@@ -245,7 +245,7 @@ pmix_status_t pmix_ptl_base_parse_uri(const char *evar, char **nspace, pmix_rank
      * stop on any FQDN or IPv4 separations */
     if (NULL == (p = strrchr(uri[0], '.'))) {
         PMIX_ERROR_LOG(PMIX_ERR_BAD_PARAM);
-        pmix_argv_free(uri);
+        PMIx_Argv_free(uri);
         return PMIX_ERR_NOT_SUPPORTED;
     }
     *p = '\0';
@@ -257,7 +257,7 @@ pmix_status_t pmix_ptl_base_parse_uri(const char *evar, char **nspace, pmix_rank
         *suri = strdup(uri[1]);
     }
 
-    pmix_argv_free(uri);
+    PMIx_Argv_free(uri);
     return PMIX_SUCCESS;
 }
 
@@ -516,16 +516,15 @@ pmix_status_t pmix_ptl_base_setup_connection(char *uri, struct sockaddr_storage
     return PMIX_SUCCESS;
 }
 
-static pmix_status_t send_connect_ack(pmix_peer_t *peer, pmix_info_t iptr[], size_t niptr)
+static pmix_status_t send_connect_ack(pmix_peer_t *peer,
+                                      pmix_info_t iptr[], size_t niptr)
 {
     char *msg;
     size_t sdsize = 0;
     pmix_status_t rc;
 
-    pmix_output_verbose(2, pmix_ptl_base_framework.framework_output, "pmix:ptl SEND CONNECT ACK");
-
-    /* allow space for a marker indicating client vs tool */
-    sdsize = 1;
+    pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
+                        "pmix:ptl SEND CONNECT ACK");
 
     /* set our ID flag and compute the required handshake size */
     peer->proc_type.flag = pmix_ptl_base_set_flag(&sdsize);
@@ -533,6 +532,7 @@ static pmix_status_t send_connect_ack(pmix_peer_t *peer, pmix_info_t iptr[], siz
     /* construct the contact message */
     rc = pmix_ptl_base_construct_message(peer, &msg, &sdsize, iptr, niptr);
     if (PMIX_SUCCESS != rc) {
+        PMIX_ERROR_LOG(rc);
         return rc;
     }
 
@@ -599,8 +599,8 @@ static pmix_status_t recv_connect_ack(pmix_peer_t *peer)
     return PMIX_SUCCESS;
 }
 
-pmix_status_t pmix_ptl_base_make_connection(pmix_peer_t *peer, char *suri, pmix_info_t *iptr,
-                                            size_t niptr)
+pmix_status_t pmix_ptl_base_make_connection(pmix_peer_t *peer, char *suri,
+                                            pmix_info_t *iptr, size_t niptr)
 {
     struct sockaddr_storage myconnection;
     pmix_status_t rc;
@@ -722,6 +722,12 @@ pmix_rnd_flag_t pmix_ptl_base_set_flag(size_t *sz)
             }
         }
 
+    } else if (PMIX_PEER_IS_SCHEDULER(pmix_globals.mypeer)) {
+        /* add space for our uid/gid for ACL purposes */
+        sdsize += 2 * sizeof(uint32_t);
+        flag = PMIX_SCHEDULER_WITH_ID;
+        sdsize += strlen(pmix_globals.myid.nspace) + 1 + sizeof(uint32_t);
+
     } else if (PMIX_PEER_IS_CLIENT(pmix_globals.mypeer)
                && !PMIX_PEER_IS_TOOL(pmix_globals.mypeer)) {
         if (PMIX_PEER_IS_SINGLETON(pmix_globals.mypeer)) {
@@ -879,6 +885,7 @@ pmix_status_t pmix_ptl_base_construct_message(pmix_peer_t *peer, char **msgout,
 
     case PMIX_TOOL_GIVEN_ID:
     case PMIX_LAUNCHER_GIVEN_ID:
+    case PMIX_SCHEDULER_WITH_ID:
     case PMIX_SINGLETON_CLIENT:
         /* self-started tool/launcher/singleton process that was given an identifier by caller */
         euid = geteuid();
@@ -932,8 +939,8 @@ pmix_status_t pmix_ptl_base_construct_message(pmix_peer_t *peer, char **msgout,
     return PMIX_SUCCESS;
 }
 
-pmix_status_t pmix_ptl_base_set_timeout(pmix_peer_t *peer, struct timeval *save, pmix_socklen_t *sz,
-                                        bool *sockopt)
+pmix_status_t pmix_ptl_base_set_timeout(pmix_peer_t *peer, struct timeval *save,
+                                        pmix_socklen_t *sz, bool *sockopt)
 {
     struct timeval tv;
 
@@ -1257,7 +1264,7 @@ complete:
     /* convert the list to an array */
     if (0 < (ndata = pmix_list_get_size(&mylist))) {
         ians = PMIX_NEW(pmix_infolist_t);
-        PMIX_LOAD_KEY(&ians->info.key, PMIX_SERVER_INFO_ARRAY);
+        PMIX_LOAD_KEY(ians->info.key, PMIX_SERVER_INFO_ARRAY);
         ians->info.value.type = PMIX_DATA_ARRAY;
         PMIX_DATA_ARRAY_CREATE(ians->info.value.data.darray, ndata, PMIX_INFO);
         sdata = (pmix_info_t *) ians->info.value.data.darray->array;
@@ -1416,12 +1423,12 @@ char **pmix_ptl_base_split_and_resolve(const char *orig_str,
         return NULL;
     }
 
-    argv = pmix_argv_split(orig_str, ',');
+    argv = PMIx_Argv_split(orig_str, ',');
     interfaces = NULL;
     for (i = 0; NULL != argv[i]; ++i) {
         if (isalpha(argv[i][0])) {
             /* This is an interface name. If not already in the interfaces array, add it */
-            pmix_argv_append_unique_nosize(&interfaces, argv[i]);
+            PMIx_Argv_append_unique_nosize(&interfaces, argv[i]);
             pmix_output_verbose(20,
                                 pmix_ptl_base_framework.framework_output,
                                 "ptl:tool: Using interface: %s ", argv[i]);
@@ -1467,7 +1474,7 @@ char **pmix_ptl_base_split_and_resolve(const char *orig_str,
                    add it. If it's already in the array, treat it as a match */
                 found = true;
                 pmix_ifindextoname(if_index, if_name, sizeof(if_name));
-                pmix_argv_append_unique_nosize(&interfaces, if_name);
+                PMIx_Argv_append_unique_nosize(&interfaces, if_name);
                 pmix_output_verbose(20,
                                     pmix_ptl_base_framework.framework_output,
                                     "ptl:base: Found match: %s (%s)",
@@ -1484,6 +1491,6 @@ char **pmix_ptl_base_split_and_resolve(const char *orig_str,
         }
     }
 
-    pmix_argv_free(argv);
+    PMIx_Argv_free(argv);
     return interfaces;
 }
