@@ -14,7 +14,7 @@
  * Copyright (c) 2014-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015-2020 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -67,8 +67,6 @@ pmix_ptl_base_t pmix_ptl_base = {
     .selected = false,
     .posted_recvs = PMIX_LIST_STATIC_INIT,
     .unexpected_msgs = PMIX_LIST_STATIC_INIT,
-    .stop_thread = {0, 0},
-    .listen_thread_active = false,
     .listener = PMIX_LISTENER_STATIC_INIT,
     .connection = NULL,
     .current_tag = 0,
@@ -78,6 +76,7 @@ pmix_ptl_base_t pmix_ptl_base = {
     .report_uri = NULL,
     .uri = NULL,
     .urifile = NULL,
+    .scheduler_filename = NULL,
     .system_filename = NULL,
     .session_filename = NULL,
     .nspace_filename = NULL,
@@ -86,6 +85,7 @@ pmix_ptl_base_t pmix_ptl_base = {
     .created_rendezvous_file = false,
     .created_session_tmpdir = false,
     .created_system_tmpdir = false,
+    .created_scheduler_filename = false,
     .created_system_filename = false,
     .created_session_filename = false,
     .created_nspace_filename = false,
@@ -132,21 +132,23 @@ static int pmix_ptl_register(pmix_mca_base_register_flag_t flags)
                                &max_msg_size);
     pmix_ptl_base.max_msg_size = max_msg_size * 1024 * 1024;
 
-    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "if_include",
-                                     "Comma-delimited list of devices and/or CIDR notation of TCP networks "
-                                     "(e.g., \"eth0,192.168.0.0/16\").  Mutually exclusive with ptl_tcp_if_exclude.",
-                                     PMIX_MCA_BASE_VAR_TYPE_STRING,
-                                     &pmix_ptl_base.if_include);
+    idx = pmix_mca_base_var_register(
+        "pmix", "ptl", "base", "if_include",
+        "Comma-delimited list of devices and/or CIDR notation of TCP networks "
+        "(e.g., \"eth0,192.168.0.0/16\").  Mutually exclusive with ptl_tcp_if_exclude.",
+        PMIX_MCA_BASE_VAR_TYPE_STRING,
+        &pmix_ptl_base.if_include);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "if_include",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "if_exclude",
-                                     "Comma-delimited list of devices and/or CIDR notation of TCP networks to NOT use "
-                                     "-- all devices not matching these specifications will be used (e.g., "
-                                     "\"eth0,192.168.0.0/16\"). "
-                                     "If set to a non-default value, it is mutually exclusive with ptl_tcp_if_include.",
-                                     PMIX_MCA_BASE_VAR_TYPE_STRING,
-                                     &pmix_ptl_base.if_exclude);
+    idx = pmix_mca_base_var_register(
+        "pmix", "ptl", "base", "if_exclude",
+        "Comma-delimited list of devices and/or CIDR notation of TCP networks to NOT use "
+        "-- all devices not matching these specifications will be used (e.g., "
+        "\"eth0,192.168.0.0/16\"). "
+        "If set to a non-default value, it is mutually exclusive with ptl_tcp_if_include.",
+        PMIX_MCA_BASE_VAR_TYPE_STRING,
+        &pmix_ptl_base.if_exclude);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "if_exclude",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
@@ -170,8 +172,7 @@ static int pmix_ptl_register(pmix_mca_base_register_flag_t flags)
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
     idx = pmix_mca_base_var_register("pmix", "ptl", "base", "disable_ipv4_family",
-                                     "Disable the IPv4 interfaces",
-                                     PMIX_MCA_BASE_VAR_TYPE_BOOL,
+                                     "Disable the IPv4 interfaces", PMIX_MCA_BASE_VAR_TYPE_BOOL,
                                      &pmix_ptl_base.disable_ipv4_family);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "disable_ipv4_family",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
@@ -184,31 +185,35 @@ static int pmix_ptl_register(pmix_mca_base_register_flag_t flags)
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "disable_ipv6_family",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "connection_wait_time",
-                                     "Number of seconds to wait for the server connection file to appear",
-                                     PMIX_MCA_BASE_VAR_TYPE_INT,
-                                     &pmix_ptl_base.wait_to_connect);
+    idx = pmix_mca_base_var_register(
+        "pmix", "ptl", "base", "connection_wait_time",
+        "Number of seconds to wait for the server connection file to appear",
+        PMIX_MCA_BASE_VAR_TYPE_INT,
+        &pmix_ptl_base.wait_to_connect);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "connection_wait_time",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "max_retries",
-                                     "Number of times to look for the connection file before quitting",
-                                     PMIX_MCA_BASE_VAR_TYPE_INT,
-                                     &pmix_ptl_base.max_retries);
+    idx = pmix_mca_base_var_register(
+        "pmix", "ptl", "base", "max_retries",
+        "Number of times to look for the connection file before quitting",
+        PMIX_MCA_BASE_VAR_TYPE_INT,
+        &pmix_ptl_base.max_retries);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "max_retries",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "handshake_wait_time",
-                                     "Number of seconds to wait for the server reply to the handshake request",
-                                     PMIX_MCA_BASE_VAR_TYPE_INT,
-                                     &pmix_ptl_base.handshake_wait_time);
+    idx = pmix_mca_base_var_register(
+        "pmix", "ptl", "base", "handshake_wait_time",
+        "Number of seconds to wait for the server reply to the handshake request",
+        PMIX_MCA_BASE_VAR_TYPE_INT,
+        &pmix_ptl_base.handshake_wait_time);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "handshake_wait_time",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
-    idx = pmix_mca_base_var_register("pmix", "ptl", "base", "handshake_max_retries",
-                                     "Number of times to retry the handshake request before giving up",
-                                     PMIX_MCA_BASE_VAR_TYPE_INT,
-                                     &pmix_ptl_base.handshake_max_retries);
+    idx = pmix_mca_base_var_register(
+        "pmix", "ptl", "base", "handshake_max_retries",
+        "Number of times to retry the handshake request before giving up",
+        PMIX_MCA_BASE_VAR_TYPE_INT,
+        &pmix_ptl_base.handshake_max_retries);
     (void) pmix_mca_base_var_register_synonym(idx, "pmix", "ptl", "tcp", "handshake_max_retries",
                                               PMIX_MCA_BASE_VAR_SYN_FLAG_DEPRECATED);
 
@@ -249,6 +254,17 @@ static pmix_status_t pmix_ptl_close(void)
     PMIX_LIST_DESTRUCT(&pmix_ptl_base.unexpected_msgs);
     PMIX_DESTRUCT(&pmix_ptl_base.listener);
 
+    if (NULL != pmix_ptl_base.scheduler_filename) {
+        if (pmix_ptl_base.created_scheduler_filename) {
+            rc = remove(pmix_ptl_base.scheduler_filename);
+            if (0 != rc) {
+                pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
+                                    "Remove of %s failed: %s",
+                                    pmix_ptl_base.scheduler_filename, strerror(errno));
+            }
+        }
+        free(pmix_ptl_base.scheduler_filename);
+    }
     if (NULL != pmix_ptl_base.system_filename) {
         if (pmix_ptl_base.created_system_filename) {
             rc = remove(pmix_ptl_base.system_filename);
@@ -346,7 +362,6 @@ static pmix_status_t pmix_ptl_open(pmix_mca_base_open_flag_t flags)
     pmix_ptl_base.initialized = true;
     PMIX_CONSTRUCT(&pmix_ptl_base.posted_recvs, pmix_list_t);
     PMIX_CONSTRUCT(&pmix_ptl_base.unexpected_msgs, pmix_list_t);
-    pmix_ptl_base.listen_thread_active = false;
     PMIX_CONSTRUCT(&pmix_ptl_base.listener, pmix_listener_t);
     pmix_ptl_base.current_tag = PMIX_PTL_TAG_DYNAMIC;
     pmix_ptl_base.connection = (struct sockaddr_storage *)malloc(sizeof(struct sockaddr_storage));
@@ -415,7 +430,9 @@ static void sdes(pmix_ptl_send_t *p)
         PMIX_RELEASE(p->data);
     }
 }
-PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_ptl_send_t, pmix_list_item_t, scon, sdes);
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_ptl_send_t,
+                                pmix_list_item_t,
+                                scon, sdes);
 
 static void rcon(pmix_ptl_recv_t *p)
 {
@@ -434,7 +451,9 @@ static void rdes(pmix_ptl_recv_t *p)
         PMIX_RELEASE(p->peer);
     }
 }
-PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_ptl_recv_t, pmix_list_item_t, rcon, rdes);
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_ptl_recv_t,
+                                pmix_list_item_t,
+                                rcon, rdes);
 
 static void prcon(pmix_ptl_posted_recv_t *p)
 {
@@ -442,7 +461,9 @@ static void prcon(pmix_ptl_posted_recv_t *p)
     p->cbfunc = NULL;
     p->cbdata = NULL;
 }
-PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_ptl_posted_recv_t, pmix_list_item_t, prcon, NULL);
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_ptl_posted_recv_t,
+                                pmix_list_item_t,
+                                prcon, NULL);
 
 static void srcon(pmix_ptl_sr_t *p)
 {
@@ -457,7 +478,9 @@ static void srdes(pmix_ptl_sr_t *p)
         PMIX_RELEASE(p->peer);
     }
 }
-PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_ptl_sr_t, pmix_object_t, srcon, srdes);
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_ptl_sr_t,
+                                pmix_object_t,
+                                srcon, srdes);
 
 static void pccon(pmix_pending_connection_t *p)
 {
@@ -498,10 +521,14 @@ static void pcdes(pmix_pending_connection_t *p)
         free(p->cred);
     }
 }
-PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_pending_connection_t, pmix_object_t, pccon, pcdes);
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_pending_connection_t,
+                                pmix_object_t,
+                                pccon, pcdes);
 
 static void lcon(pmix_listener_t *p)
 {
+    memset(&p->ev, 0, sizeof(pmix_event_t));
+    p->active = false;
     p->socket = -1;
     p->varname = NULL;
     p->uri = NULL;
@@ -521,7 +548,9 @@ static void ldes(pmix_listener_t *p)
         free(p->uri);
     }
 }
-PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_listener_t, pmix_list_item_t, lcon, ldes);
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_listener_t,
+                                pmix_list_item_t,
+                                lcon, ldes);
 
 static void qcon(pmix_ptl_queue_t *p)
 {
@@ -535,7 +564,9 @@ static void qdes(pmix_ptl_queue_t *p)
         PMIX_RELEASE(p->peer);
     }
 }
-PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_ptl_queue_t, pmix_object_t, qcon, qdes);
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_ptl_queue_t,
+                                pmix_object_t,
+                                qcon, qdes);
 
 static void ccon(pmix_connection_t *p)
 {
@@ -557,4 +588,6 @@ static void dcon(pmix_connection_t *p)
         free(p->version);
     }
 }
-PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_connection_t, pmix_list_item_t, ccon, dcon);
+PMIX_EXPORT PMIX_CLASS_INSTANCE(pmix_connection_t,
+                                pmix_list_item_t,
+                                ccon, dcon);
