@@ -8,7 +8,7 @@
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
- * Copyright (c) 2021      Nanook Consulting  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -49,26 +49,29 @@
 #include <pthread.h>
 
 #include "src/class/pmix_list.h"
-#include "src/util/basename.h"
-#include "src/util/error.h"
-#include "src/util/fd.h"
-#include "src/util/net.h"
-#include "src/util/os_dirpath.h"
-#include "src/util/output.h"
-#include "src/util/pif.h"
+#include "src/include/pmix_socket_errno.h"
+#include "src/util/pmix_argv.h"
+#include "src/util/pmix_basename.h"
+#include "src/util/pmix_error.h"
+#include "src/util/pmix_fd.h"
+#include "src/util/pmix_net.h"
+#include "src/util/pmix_os_dirpath.h"
+#include "src/util/pmix_output.h"
+#include "src/util/pmix_if.h"
 #include "src/util/pmix_environ.h"
-#include "src/util/printf.h"
-#include "src/util/show_help.h"
+#include "src/util/pmix_printf.h"
+#include "src/util/pmix_show_help.h"
 
 #include "src/mca/ptl/base/base.h"
 
-// local functions for connection support
-static void *listen_thread(void *obj);
-static pthread_t engine;
+// local connection handler
+static void connection_event_handler(int incoming_sd, short flags, void *cbdata);
+
+// local value for connection support
 static bool setup_complete = false;
 
 /*
- * start listening thread
+ * start listening event
  */
 pmix_status_t pmix_ptl_base_start_listening(pmix_info_t info[], size_t ninfo)
 {
@@ -83,156 +86,112 @@ pmix_status_t pmix_ptl_base_start_listening(pmix_info_t info[], size_t ninfo)
     }
     setup_complete = true;
 
-    /*** spawn internal listener thread */
-    if (0 > pipe(pmix_ptl_base.stop_thread)) {
-        PMIX_ERROR_LOG(PMIX_ERR_IN_ERRNO);
-        return PMIX_ERR_OUT_OF_RESOURCE;
-    }
-    /* Make sure the pipe FDs are set to close-on-exec so that
-       they don't leak into children */
-    if (pmix_fd_set_cloexec(pmix_ptl_base.stop_thread[0]) != PMIX_SUCCESS
-        || pmix_fd_set_cloexec(pmix_ptl_base.stop_thread[1]) != PMIX_SUCCESS) {
-        PMIX_ERROR_LOG(PMIX_ERR_IN_ERRNO);
-        close(pmix_ptl_base.stop_thread[0]);
-        close(pmix_ptl_base.stop_thread[1]);
-        return PMIX_ERR_OUT_OF_RESOURCE;
-    }
-    /* fork off the listener thread */
-    pmix_ptl_base.listen_thread_active = true;
-    if (0 > pthread_create(&engine, NULL, listen_thread, NULL)) {
-        pmix_ptl_base.listen_thread_active = false;
-        return PMIX_ERROR;
-    }
+    pmix_event_set(pmix_globals.evbase, &pmix_ptl_base.listener.ev,
+               pmix_ptl_base.listener.socket,
+               PMIX_EV_READ|PMIX_EV_PERSIST,
+               connection_event_handler, 0);
+    pmix_ptl_base.listener.active = true;
+    pmix_event_add(&pmix_ptl_base.listener.ev, 0);
 
     return PMIX_SUCCESS;
 }
 
 void pmix_ptl_base_stop_listening(void)
 {
-    int i;
     pmix_listener_t *lt = &pmix_ptl_base.listener;
 
-    pmix_output_verbose(8, pmix_ptl_base_framework.framework_output, "listen_thread: shutdown");
+    pmix_output_verbose(8, pmix_ptl_base_framework.framework_output,
+                        "listen_thread: shutdown");
 
-    if (!pmix_ptl_base.listen_thread_active) {
-        /* nothing we can do */
+    if (!lt->active) {
+        /* nothing we need do */
         return;
     }
 
     /* mark it as inactive */
-    pmix_ptl_base.listen_thread_active = false;
-    /* use the block to break it loose just in
-     * case the thread is blocked in a call to select for
-     * a long time */
-    i = 1;
-    if (0 > write(pmix_ptl_base.stop_thread[1], &i, sizeof(int))) {
-        return;
-    }
-    /* wait for thread to exit */
-    pthread_join(engine, NULL);
+    lt->active = false;
+    pmix_event_del(&lt->ev);
     /* close the socket to remove the connection points */
     CLOSE_THE_SOCKET(lt->socket);
     lt->socket = -1;
 }
 
-static void *listen_thread(void *obj)
+/*
+ * Handler for accepting connections from the event library
+ */
+static void connection_event_handler(int incoming_sd, short flags, void *cbdata)
 {
-    (void) obj;
-    int rc, max;
-    socklen_t addrlen = sizeof(struct sockaddr_storage);
+    struct sockaddr addr;
+    pmix_socklen_t addrlen = sizeof(struct sockaddr);
+    int sd;
     pmix_pending_connection_t *pending_connection;
-    struct timeval timeout;
-    fd_set readfds;
     pmix_listener_t *lt = &pmix_ptl_base.listener;
+    PMIX_HIDE_UNUSED_PARAMS(flags, cbdata);
 
-    pmix_output_verbose(8, pmix_ptl_base_framework.framework_output, "listen_thread: active");
-
-    while (pmix_ptl_base.listen_thread_active) {
-        FD_ZERO(&readfds);
-        FD_SET(lt->socket, &readfds);
-        max = lt->socket;
-
-        /* add the stop_thread fd */
-        FD_SET(pmix_ptl_base.stop_thread[0], &readfds);
-        max = (pmix_ptl_base.stop_thread[0] > max) ? pmix_ptl_base.stop_thread[0] : max;
-
-        /* set timeout interval */
-        timeout.tv_sec = 2;
-        timeout.tv_usec = 0;
-
-        /* Block in a select to avoid hammering the cpu.  If a connection
-         * comes in, we'll get woken up right away.
-         */
-        rc = select(max + 1, &readfds, NULL, NULL, &timeout);
-        if (!pmix_ptl_base.listen_thread_active) {
-            /* we've been asked to terminate */
-            close(pmix_ptl_base.stop_thread[0]);
-            close(pmix_ptl_base.stop_thread[1]);
-            return NULL;
-        }
-        if (rc < 0) {
-            continue;
+    sd = accept(incoming_sd, (struct sockaddr *) &addr, &addrlen);
+    pmix_output_verbose(5, pmix_ptl_base_framework.framework_output,
+                        "connection_event_handler: working connection "
+                        "(%d, %d) %s:%d\n",
+                        sd, pmix_socket_errno,
+                        pmix_net_get_hostname((struct sockaddr *) &addr),
+                        pmix_net_get_port((struct sockaddr *) &addr));
+    if (sd < 0) {
+        /* Non-fatal errors */
+        if (EINTR == pmix_socket_errno ||
+            EAGAIN == pmix_socket_errno ||
+            EWOULDBLOCK == pmix_socket_errno) {
+            return;
         }
 
-        /* according to the man pages, select replaces the given descriptor
-         * set with a subset consisting of those descriptors that are ready
-         * for the specified operation - in this case, a read. So we need to
-         * first check to see if this file descriptor is included in the
-         * returned subset
-         */
-        if (0 == FD_ISSET(lt->socket, &readfds)) {
-            /* this descriptor is not included */
-            continue;
+        /* If we run out of file descriptors, log an extra warning (so
+           that the user can know to fix this problem) and abandon all
+           hope. */
+        else if (EMFILE == pmix_socket_errno) {
+            CLOSE_THE_SOCKET(incoming_sd);
+            PMIX_ERROR_LOG(PMIX_ERR_OUT_OF_RESOURCE);
+            pmix_show_help("help-ptl-base.txt", "accept failed", true,
+                           pmix_globals.hostname,
+                           pmix_socket_errno, strerror(pmix_socket_errno),
+                           "Out of file descriptors");
+            return;
         }
 
-        /* this descriptor is ready to be read, which means a connection
-         * request has been received - so harvest it. All we want to do
-         * here is accept the connection and push the info onto the event
-         * library for subsequent processing - we don't want to actually
-         * process the connection here as it takes too long, and so the
-         * OS might start rejecting connections due to timeout.
-         */
-        pending_connection = PMIX_NEW(pmix_pending_connection_t);
-        pending_connection->protocol = lt->protocol;
-        pmix_event_assign(&pending_connection->ev, pmix_globals.evbase, -1, EV_WRITE, lt->cbfunc,
-                          pending_connection);
-        pending_connection->sd = accept(lt->socket, (struct sockaddr *) &(pending_connection->addr),
-                                        &addrlen);
-        if (pending_connection->sd < 0) {
-            PMIX_RELEASE(pending_connection);
-            if (pmix_socket_errno != EAGAIN || pmix_socket_errno != EWOULDBLOCK) {
-                if (EMFILE == pmix_socket_errno || ENOBUFS == pmix_socket_errno
-                    || ENOMEM == pmix_socket_errno) {
-                    PMIX_ERROR_LOG(PMIX_ERR_OUT_OF_RESOURCE);
-                } else if (EINVAL == pmix_socket_errno || EINTR == pmix_socket_errno) {
-                    /* race condition at finalize */
-                    goto done;
-                } else if (ECONNABORTED == pmix_socket_errno) {
-                    /* they aborted the attempt */
-                    continue;
-                } else {
-                    pmix_output(0, "listen_thread: accept() failed: %s (%d).",
-                                strerror(pmix_socket_errno), pmix_socket_errno);
-                }
-                goto done;
-            }
-            continue;
+        /* For all other cases, close the socket, print a warning but
+           try to continue */
+        else {
+            CLOSE_THE_SOCKET(incoming_sd);
+            pmix_show_help("help-ptl-base.txt", "accept failed", true,
+                           pmix_globals.hostname,
+                           pmix_socket_errno, strerror(pmix_socket_errno),
+                           "Unknown cause; job will try to continue");
+            return;
         }
-
-        pmix_output_verbose(8, pmix_ptl_base_framework.framework_output,
-                            "listen_thread: new connection: (%d, %d)", pending_connection->sd,
-                            pmix_socket_errno);
-        /* post the object */
-        PMIX_POST_OBJECT(pending_connection);
-        /* activate the event */
-        pmix_event_active(&pending_connection->ev, EV_WRITE, 1);
     }
 
-done:
-    pmix_ptl_base.listen_thread_active = false;
-    return NULL;
+    /* this descriptor is ready to be read, which means a connection
+     * request has been received - so harvest it. All we want to do
+     * here is accept the connection and push the info onto the event
+     * library for subsequent processing - we don't want to actually
+     * process the connection here as it takes too long, and so the
+     * OS might start rejecting connections due to timeout.
+     */
+    pending_connection = PMIX_NEW(pmix_pending_connection_t);
+    pending_connection->protocol = lt->protocol;
+    pmix_event_assign(&pending_connection->ev, pmix_globals.evbase,
+                      -1, EV_WRITE,
+                      lt->cbfunc, pending_connection);
+    pending_connection->sd = sd;
+
+    pmix_output_verbose(8, pmix_ptl_base_framework.framework_output,
+                        "connection_event_handler: new connection: (%d, %d)", pending_connection->sd,
+                        pmix_socket_errno);
+    /* post the object */
+    PMIX_POST_OBJECT(pending_connection);
+    /* activate the event */
+    pmix_event_active(&pending_connection->ev, EV_WRITE, 1);
 }
 
+
 pmix_status_t pmix_base_write_rndz_file(char *filename, char *uri, bool *created)
 {
     FILE *fp;
@@ -295,7 +254,7 @@ pmix_status_t pmix_base_write_rndz_file(char *filename, char *uri, bool *created
  * If we are a server and are given a rendezvous file, then that is
  * is the name of the file we are to use to store our listener info.
  */
-pmix_status_t pmix_ptl_base_setup_listener(void)
+pmix_status_t pmix_ptl_base_setup_listener(pmix_info_t info[], size_t ninfo)
 {
     int flags = 0;
     pmix_listener_t *lt;
@@ -313,8 +272,55 @@ pmix_status_t pmix_ptl_base_setup_listener(void)
     pid_t mypid;
     int outpipe;
     char *leftover;
+    size_t n;
 
-    pmix_output_verbose(2, pmix_ptl_base_framework.framework_output, "ptl:tool setup_listener");
+    pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
+                        "ptl:tool setup_listener");
+
+    for (n = 0; n < ninfo; n++) {
+        if (0 == strcmp(info[n].key, PMIX_SERVER_SESSION_SUPPORT)) {
+            pmix_ptl_base.session_tool = PMIX_INFO_TRUE(&info[n]);
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_SERVER_SYSTEM_SUPPORT)) {
+            pmix_ptl_base.system_tool = PMIX_INFO_TRUE(&info[n]);
+        } else if (0 == strcmp(info[n].key, PMIX_SERVER_TOOL_SUPPORT)) {
+            pmix_ptl_base.tool_support = PMIX_INFO_TRUE(&info[n]);
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_SERVER_REMOTE_CONNECTIONS)) {
+            pmix_ptl_base.remote_connections = PMIX_INFO_TRUE(&info[n]);
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_TCP_IF_INCLUDE)) {
+            pmix_ptl_base.if_include = strdup(info[n].value.data.string);
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_TCP_IF_EXCLUDE)) {
+            pmix_ptl_base.if_exclude = strdup(info[n].value.data.string);
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_TCP_IPV4_PORT)) {
+            pmix_ptl_base.ipv4_port = info[n].value.data.integer;
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_TCP_IPV6_PORT)) {
+            pmix_ptl_base.ipv6_port = info[n].value.data.integer;
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_TCP_DISABLE_IPV4)) {
+            pmix_ptl_base.disable_ipv4_family = PMIX_INFO_TRUE(&info[n]);
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_TCP_DISABLE_IPV6)) {
+            pmix_ptl_base.disable_ipv6_family = PMIX_INFO_TRUE(&info[n]);
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_TCP_REPORT_URI)) {
+            if (NULL != pmix_ptl_base.report_uri) {
+                free(pmix_ptl_base.report_uri);
+            }
+            pmix_ptl_base.report_uri = strdup(info[n].value.data.string);
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_SERVER_TMPDIR)) {
+            if (NULL != pmix_ptl_base.session_tmpdir) {
+                free(pmix_ptl_base.session_tmpdir);
+            }
+            pmix_ptl_base.session_tmpdir = strdup(info[n].value.data.string);
+        } else if (PMIX_CHECK_KEY(&info[n], PMIX_SYSTEM_TMPDIR)) {
+            if (NULL != pmix_ptl_base.system_tmpdir) {
+                free(pmix_ptl_base.system_tmpdir);
+            }
+            pmix_ptl_base.system_tmpdir = strdup(info[n].value.data.string);
+        }
+    }
+
+    if (NULL != pmix_ptl_base.if_include && NULL != pmix_ptl_base.if_exclude) {
+        pmix_show_help("help-ptl-base.txt", "include-exclude", true, pmix_ptl_base.if_include,
+                       pmix_ptl_base.if_exclude);
+        return PMIX_ERR_SILENT;
+    }
 
     lt = &pmix_ptl_base.listener;
 
@@ -382,7 +388,7 @@ pmix_status_t pmix_ptl_base_setup_listener(void)
              */
             if (PMIX_ERR_FABRIC_NOT_PARSEABLE == rc) {
                 pmix_show_help("help-ptl-base.txt", "not-parseable", true);
-                pmix_argv_free(interfaces);
+                PMIx_Argv_free(interfaces);
                 return PMIX_ERR_BAD_PARAM;
             }
             /* if we are including, then ignore this if not present */
@@ -424,7 +430,7 @@ pmix_status_t pmix_ptl_base_setup_listener(void)
     }
     /* cleanup */
     if (NULL != interfaces) {
-        pmix_argv_free(interfaces);
+        PMIx_Argv_free(interfaces);
     }
 
     /* if we didn't find anything, that could be a problem */
@@ -450,15 +456,13 @@ pmix_status_t pmix_ptl_base_setup_listener(void)
 
     /* set the port */
     if (AF_INET == pmix_ptl_base.connection->ss_family) {
-        ((struct sockaddr_in *) pmix_ptl_base.connection)->sin_port = htons(
-            pmix_ptl_base.ipv4_port);
+        ((struct sockaddr_in *) pmix_ptl_base.connection)->sin_port = htons(pmix_ptl_base.ipv4_port);
         addrlen = sizeof(struct sockaddr_in);
         if (0 != pmix_ptl_base.ipv4_port) {
             flags = 1;
         }
     } else if (AF_INET6 == pmix_ptl_base.connection->ss_family) {
-        ((struct sockaddr_in6 *) pmix_ptl_base.connection)->sin6_port = htons(
-            pmix_ptl_base.ipv6_port);
+        ((struct sockaddr_in6 *) pmix_ptl_base.connection)->sin6_port = htons(pmix_ptl_base.ipv6_port);
         addrlen = sizeof(struct sockaddr_in6);
         if (0 != pmix_ptl_base.ipv6_port) {
             flags = 1;
@@ -547,10 +551,20 @@ pmix_status_t pmix_ptl_base_setup_listener(void)
     if (0 > rc || NULL == lt->uri) {
         goto sockerror;
     }
-    pmix_output_verbose(2, pmix_ptl_base_framework.framework_output, "ptl:base URI %s", lt->uri);
+    pmix_output_verbose(2, pmix_ptl_base_framework.framework_output,
+                        "ptl:base URI %s", lt->uri);
 
     /* save the URI internally so we can report it */
     urikv = PMIX_NEW(pmix_kval_t);
+    urikv->key = strdup(PMIX_MYSERVER_URI);
+    PMIX_VALUE_CREATE(urikv->value, 1);
+    PMIX_VALUE_LOAD(urikv->value, lt->uri, PMIX_STRING);
+    PMIX_GDS_STORE_KV(rc, pmix_globals.mypeer, &pmix_globals.myid, PMIX_INTERNAL, urikv);
+    PMIX_RELEASE(urikv); // maintain accounting
+
+    /* save a legacy URI internally so we can report it
+     * to older tools */
+    urikv = PMIX_NEW(pmix_kval_t);
     urikv->key = strdup(PMIX_SERVER_URI);
     PMIX_VALUE_CREATE(urikv->value, 1);
     PMIX_VALUE_LOAD(urikv->value, lt->uri, PMIX_STRING);
@@ -617,6 +631,21 @@ pmix_status_t pmix_ptl_base_setup_listener(void)
     }
 
 nextstep:
+    /* if we are the scheduler, then drop an appropriately named
+     * contact file so the system's resource manager can find us */
+    if (PMIX_PEER_IS_SCHEDULER(pmix_globals.mypeer)) {
+        if (0 > asprintf(&pmix_ptl_base.scheduler_filename, "%s/pmix.sched.%s",
+                         pmix_ptl_base.system_tmpdir, pmix_globals.hostname)) {
+            goto sockerror;
+        }
+        rc = pmix_base_write_rndz_file(pmix_ptl_base.scheduler_filename, lt->uri,
+                                       &pmix_ptl_base.created_system_tmpdir);
+        if (PMIX_SUCCESS != rc) {
+            goto sockerror;
+        }
+        pmix_ptl_base.created_scheduler_filename = true;
+    }
+
     /* if we are going to support tools, then drop contact file(s) */
     if (pmix_ptl_base.system_tool) {
         if (0 > asprintf(&pmix_ptl_base.system_filename, "%s/pmix.sys.%s",
