@@ -15,8 +15,8 @@
  * Copyright (c) 2013-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2016-2019 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
- * Copyright (c) 2021      Triad National Security, LLC. All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2023 Triad National Security, LLC. All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -48,6 +48,7 @@
 #include "src/util/pmix_keyval_parse.h"
 #include "src/util/pmix_output.h"
 #include "src/util/pmix_show_help.h"
+#include "src/util/pmix_net.h"
 #include "src/runtime/pmix_init_util.h"
 #include <event.h>
 
@@ -62,6 +63,7 @@ void pmix_rte_finalize(void)
     int i;
     pmix_notify_caddy_t *cd;
     pmix_iof_req_t *req;
+    pmix_regattr_input_t *p;
 
     if (!pmix_init_called) {
         return;
@@ -95,6 +97,9 @@ void pmix_rte_finalize(void)
     /* close GDS */
     (void) pmix_mca_base_framework_close(&pmix_gds_base_framework);
 
+    /* Finalize the network helper subsystem. */
+    (void)pmix_net_finalize();
+
     /* finalize the mca */
     /* Clear out all the registered MCA params */
     pmix_deregister_params();
@@ -129,9 +134,8 @@ void pmix_rte_finalize(void)
     }
     PMIX_DESTRUCT(&pmix_globals.notifications);
     for (i = 0; i < pmix_globals.iof_requests.size; i++) {
-        if (NULL
-            != (req = (pmix_iof_req_t *) pmix_pointer_array_get_item(&pmix_globals.iof_requests,
-                                                                     i))) {
+        req = (pmix_iof_req_t *) pmix_pointer_array_get_item(&pmix_globals.iof_requests, i);
+        if (NULL != req) {
             PMIX_RELEASE(req);
         }
     }
@@ -142,6 +146,25 @@ void pmix_rte_finalize(void)
         pmix_globals.hostname = NULL;
     }
     PMIX_LIST_DESTRUCT(&pmix_globals.nspaces);
+    PMIX_LIST_DESTRUCT(&pmix_client_globals.groups);
+
+    for (i=0; i < pmix_globals.keyindex.size; i++) {
+        p = (pmix_regattr_input_t*)pmix_pointer_array_get_item(&pmix_globals.keyindex, i);
+        if (NULL != p) {
+            if (NULL != p->name) {
+                free(p->name);
+            }
+            if (NULL != p->string) {
+                free(p->string);
+            }
+            if (NULL != p->description) {
+                PMIx_Argv_free(p->description);
+            }
+            free(p);
+        }
+    }
+    PMIX_DESTRUCT(&pmix_globals.keyindex);
+    free(pmix_globals.myidval.data.proc);
 
     /* now safe to release the event base */
     (void) pmix_progress_thread_stop(NULL);
