@@ -18,7 +18,7 @@
  * Copyright (c) 2013-2020 Intel, Inc.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
  * Copyright (c) 2022      Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
@@ -91,6 +91,7 @@ PMIX_EXPORT pmix_globals_t pmix_globals = {
     .sessionid = UINT32_MAX,
     .pindex = 0,
     .evbase = NULL,
+    .evauxbase = NULL,
     .debug_output = -1,
     .events = PMIX_EVENTS_STATIC_INIT,
     .connected = false,
@@ -262,6 +263,8 @@ int pmix_rte_init(uint32_t type, pmix_info_t info[], size_t ninfo, pmix_ptl_cbfu
                 }
             } else if (PMIX_CHECK_KEY(&info[n], PMIX_EXTERNAL_PROGRESS)) {
                 pmix_globals.external_progress = PMIX_INFO_TRUE(&info[n]);
+            } else if (PMIX_CHECK_KEY(&info[n], PMIX_EXTERNAL_AUX_EVENT_BASE)) {
+                pmix_globals.evauxbase = (pmix_event_base_t*)info[n].value.data.ptr;
             } else if (PMIX_CHECK_KEY(&info[n], PMIX_HOSTNAME_KEEP_FQDN)) {
                 keepfqdn = PMIX_INFO_TRUE(&info[n]);
             } else if (PMIX_CHECK_KEY(&info[n], PMIX_BIND_PROGRESS_THREAD)) {
@@ -285,6 +288,10 @@ int pmix_rte_init(uint32_t type, pmix_info_t info[], size_t ninfo, pmix_ptl_cbfu
         ret = PMIX_ERROR;
         goto return_error;
     }
+    /* if we were not given an aux event base, set it to our internal one */
+    if (NULL == pmix_globals.evauxbase) {
+        pmix_globals.evauxbase = pmix_globals.evbase;
+    }
 
     /* setup the globals structure */
     pmix_globals.pid = getpid();
@@ -308,6 +315,7 @@ int pmix_rte_init(uint32_t type, pmix_info_t info[], size_t ninfo, pmix_ptl_cbfu
     PMIX_CONSTRUCT(&pmix_globals.nspaces, pmix_list_t);
     PMIX_CONSTRUCT(&pmix_globals.keyindex, pmix_pointer_array_t);
     pmix_pointer_array_init(&pmix_globals.keyindex, 1024, INT_MAX, 128);
+    PMIX_CONSTRUCT(&pmix_client_globals.groups, pmix_list_t);
     /* need to hold off checking the hotel init return code
      * until after we construct all the globals so they can
      * correctly finalize */
@@ -479,7 +487,7 @@ int pmix_rte_init(uint32_t type, pmix_info_t info[], size_t ninfo, pmix_ptl_cbfu
     /* open the psec and select the active plugins */
     if (NULL != (evar = getenv("PMIX_SECURITY_MODE"))) {
         /* convert to an MCA param, but don't overwrite something already there */
-        pmix_setenv("PMIX_MCA_psec", evar, false, &environ);
+        PMIx_Setenv("PMIX_MCA_psec", evar, false, &environ);
     }
     ret = pmix_mca_base_framework_open(&pmix_psec_base_framework,
                                        PMIX_MCA_BASE_OPEN_DEFAULT);
