@@ -8,7 +8,7 @@
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -116,7 +116,8 @@ static pmix_status_t defer_response(char *nspace, pmix_rank_t rank, pmix_server_
 
     if (localonly) {
         /* the client asked that we not wait, so return now */
-        pmix_output_verbose(2, pmix_server_globals.get_output, "%s:%d CLIENT REQUESTED IMMEDIATE",
+        pmix_output_verbose(2, pmix_server_globals.get_output,
+                            "%s:%d CLIENT REQUESTED IMMEDIATE",
                             pmix_globals.myid.nspace, pmix_globals.myid.rank);
         return PMIX_ERR_NOT_AVAILABLE;
     }
@@ -139,6 +140,7 @@ static pmix_status_t defer_response(char *nspace, pmix_rank_t rank, pmix_server_
     *locald = lcd;
     return rc;
 }
+
 pmix_status_t pmix_server_get(pmix_buffer_t *buf, pmix_modex_cbfunc_t cbfunc, void *cbdata)
 {
     pmix_server_caddy_t *cd = (pmix_server_caddy_t *) cbdata;
@@ -158,6 +160,8 @@ pmix_status_t pmix_server_get(pmix_buffer_t *buf, pmix_modex_cbfunc_t cbfunc, vo
     struct timeval tv = {0, 0};
     pmix_buffer_t pbkt;
     pmix_cb_t cb;
+    pmix_kval_t *kval;
+    pmix_byte_object_t bo;
     pmix_proc_t proc;
     char *data;
     size_t sz, n;
@@ -165,7 +169,8 @@ pmix_status_t pmix_server_get(pmix_buffer_t *buf, pmix_modex_cbfunc_t cbfunc, vo
     pmix_scope_t scope = PMIX_SCOPE_UNDEF;
     pmix_rank_info_t *iptr;
 
-    pmix_output_verbose(2, pmix_server_globals.get_output, "%s recvd GET",
+    pmix_output_verbose(2, pmix_server_globals.get_output,
+                        "%s recvd GET",
                         PMIX_NAME_PRINT(&pmix_globals.myid));
 
     /* setup */
@@ -186,6 +191,8 @@ pmix_status_t pmix_server_get(pmix_buffer_t *buf, pmix_modex_cbfunc_t cbfunc, vo
         PMIX_ERROR_LOG(rc);
         return rc;
     }
+    PMIX_LOAD_PROCID(&proc, nspace, rank);
+
     /* retrieve any provided info structs */
     cnt = 1;
     PMIX_BFROPS_UNPACK(rc, cd->peer, buf, &cd->ninfo, &cnt, PMIX_SIZE);
@@ -233,6 +240,86 @@ pmix_status_t pmix_server_get(pmix_buffer_t *buf, pmix_modex_cbfunc_t cbfunc, vo
         }
     }
 
+    /* check for a request for pset names - these are not associated
+     * with a given nspace. Instead, we are searching for any psets
+     * that contain the calling process */
+    if (keyprovided && PMIx_Check_key(key, PMIX_PSET_NAMES)) {
+        /* loop over all known psets and collect names
+         * in which this proc is a member */
+        pmix_pset_t *pset;
+        char **psets = NULL;
+        PMIX_LIST_FOREACH(pset, &pmix_server_globals.psets, pmix_pset_t) {
+            for (n=0; n < pset->nmembers; n++) {
+                if (PMIx_Check_procid(&pset->members[n], &proc)) {
+                    PMIx_Argv_append_nosize(&psets, pset->name);
+                    break;
+                }
+            }
+        }
+        if (NULL != psets) {
+            data = PMIx_Argv_join(psets, ',');
+            PMIx_Argv_free(psets);
+            // pass it back
+            // we have to assemble this data into a form that
+            // the client_get function can properly unpack
+            PMIX_CONSTRUCT(&pbkt, pmix_buffer_t);
+            PMIX_CONSTRUCT(&cb, pmix_cb_t);
+            PMIX_KVAL_NEW(kval, PMIX_PSET_NAMES);
+            kval->value->data.string = data;
+            kval->value->type = PMIX_STRING;
+            pmix_list_append(&cb.kvs, &kval->super);
+            /* assemble the provided data into a byte object */
+            PMIX_GDS_ASSEMB_KVS_REQ(rc, pmix_globals.mypeer, &proc, &cb.kvs, &pbkt, cd);
+            if (rc != PMIX_SUCCESS) {
+                PMIX_ERROR_LOG(rc);
+                PMIX_DESTRUCT(&pbkt);
+                PMIX_DESTRUCT(&cb);
+                return rc;
+            }
+            PMIX_DESTRUCT(&cb);
+            if (PMIX_PEER_IS_V1(cd->peer)) {
+                /* if the client is using v1, then it expects the
+                 * data returned to it as the rank followed by a byte object containing
+                 * a buffer - so we have to do a little gyration */
+                pmix_buffer_t xfer;
+                PMIX_CONSTRUCT(&xfer, pmix_buffer_t);
+                PMIX_BFROPS_PACK(rc, cd->peer, &xfer, &pbkt, 1, PMIX_BUFFER);
+                if (PMIX_SUCCESS != rc) {
+                    PMIX_ERROR_LOG(rc);
+                    PMIX_DESTRUCT(&pbkt);
+                    PMIX_DESTRUCT(&xfer);
+                    PMIX_DESTRUCT(&cb);
+                    return rc;
+                }
+                PMIX_UNLOAD_BUFFER(&xfer, bo.bytes, bo.size);
+                PMIX_DESTRUCT(&xfer);
+            } else {
+                PMIX_UNLOAD_BUFFER(&pbkt, bo.bytes, bo.size);
+            }
+            PMIX_DESTRUCT(&pbkt);
+            /* pack it for transmission */
+            PMIX_CONSTRUCT(&pbkt, pmix_buffer_t);
+            PMIX_BFROPS_PACK(rc, cd->peer, &pbkt, &bo, 1, PMIX_BYTE_OBJECT);
+            if (PMIX_SUCCESS != rc) {
+                PMIX_ERROR_LOG(rc);
+                PMIX_DESTRUCT(&pbkt);
+                return rc;
+            }
+            /* unload the resulting payload */
+            PMIX_UNLOAD_BUFFER(&pbkt, data, sz);
+            PMIX_DESTRUCT(&pbkt);
+            /* call the internal callback function - it will
+             * release the cbdata */
+            cbfunc(PMIX_SUCCESS, data, sz, cbdata, relfn, data);
+            /* return success so the server doesn't duplicate
+             * the release of cbdata */
+            return PMIX_SUCCESS;
+        } else {
+            // return not found as this proc doesn't belong to any psets
+            return PMIX_ERR_NOT_FOUND;
+        }
+    }
+
     /* find the nspace object for the target proc */
     nptr = NULL;
     PMIX_LIST_FOREACH (ns, &pmix_globals.nspaces, pmix_namespace_t) {
@@ -700,7 +787,7 @@ static pmix_status_t get_job_data(char *nspace,
         }
         if (PMIX_PEER_IS_V1(cd->peer)) {
             /* if the client is using v1, then it expects the
-             * data returned to it as the rank followed by abyte object containing
+             * data returned to it as the rank followed by a byte object containing
              * a buffer - so we have to do a little gyration */
             pmix_buffer_t xfer;
             PMIX_CONSTRUCT(&xfer, pmix_buffer_t);
@@ -737,7 +824,6 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
 {
     pmix_status_t rc;
     bool found = false;
-    bool used_mypeer = true;
     pmix_buffer_t pbkt, pkt;
     pmix_proc_t proc;
     pmix_cb_t cb;
@@ -790,7 +876,6 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
         }
         if (NULL != peer) {
             PMIX_GDS_FETCH_KV(rc, peer, &cb);
-            used_mypeer = false;
         }
     }
     cb.info = NULL;
@@ -799,7 +884,7 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
         found = true;
         PMIX_CONSTRUCT(&pkt, pmix_buffer_t);
         /* assemble the provided data into a byte object */
-        if (PMIX_RANK_UNDEF == rank || diffnspace || used_mypeer) {
+        if (PMIX_RANK_UNDEF == rank || diffnspace) {
             PMIX_GDS_ASSEMB_KVS_REQ(rc, pmix_globals.mypeer, &proc, &cb.kvs, &pkt, cd);
         } else {
             PMIX_GDS_ASSEMB_KVS_REQ(rc, cd->peer, &proc, &cb.kvs, &pkt, cd);
@@ -860,7 +945,7 @@ static pmix_status_t _satisfy_request(pmix_namespace_t *nptr, pmix_rank_t rank,
         /* pass it back */
         cbfunc(rc, data, sz, cbdata, relfn, data);
         // Safe to free pkt.
-        PMIX_DATA_BUFFER_DESTRUCT(&pkt);
+        PMIX_DESTRUCT(&pkt);
         return rc;
     }
 
