@@ -8,7 +8,7 @@
  * Copyright (c) 2016-2020 IBM Corporation.  All rights reserved.
  * Copyright (c) 2016-2018 Research Organization for Information Science
  *                         and Technology (RIST).  All rights reserved.
- * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  */
 
@@ -21,14 +21,14 @@
 #endif
 
 #include "src/include/pmix_config.h"
-#include "include/pmix_common.h"
-#include "src/include/types.h"
+#include "pmix_common.h"
+#include "pmix_server.h"
 
-#include "include/pmix_server.h"
 #include "src/class/pmix_hotel.h"
 #include "src/include/pmix_globals.h"
-#include "src/threads/threads.h"
-#include "src/util/hash.h"
+#include "src/include/pmix_types.h"
+#include "src/threads/pmix_threads.h"
+#include "src/util/pmix_hash.h"
 
 #define PMIX_IOF_HOTEL_SIZE 256
 #define PMIX_IOF_MAX_STAY   300000000
@@ -144,14 +144,6 @@ typedef struct {
 } pmix_regevents_info_t;
 PMIX_CLASS_DECLARATION(pmix_regevents_info_t);
 
-typedef struct {
-    pmix_list_item_t super;
-    char *grpid;
-    pmix_proc_t *members;
-    size_t nmbrs;
-} pmix_group_t;
-PMIX_CLASS_DECLARATION(pmix_group_t);
-
 typedef struct {
     pmix_list_item_t super;
     pmix_group_t *grp;
@@ -188,7 +180,7 @@ typedef struct {
     pmix_list_t gdata;  // cache of data given to me for passing to all clients
     char **genvars;     // argv array of envars given to me for passing to all clients
     pmix_list_t events; // list of pmix_regevents_info_t registered events
-    pmix_list_t groups; // list of pmix_group_t group memberships
+    char **failedgrps;    // group IDs that failed to construct
     pmix_list_t iof;    // IO to be forwarded to clients
     pmix_list_t iof_residuals;  // leftover bytes waiting for newline
     pmix_list_t psets;  // list of known psets and memberships
@@ -221,6 +213,9 @@ typedef struct {
     // verbosity for basic server functions
     int base_output;
     int base_verbose;
+    // verbosity for server group operations
+    int group_output;
+    int group_verbose;
 } pmix_server_globals_t;
 
 #define PMIX_GDS_CADDY(c, p, t)              \
@@ -248,7 +243,8 @@ PMIX_EXPORT bool pmix_server_trk_update(pmix_server_trkr_t *trk);
 
 PMIX_EXPORT void pmix_pending_nspace_requests(pmix_namespace_t *nptr);
 PMIX_EXPORT pmix_status_t pmix_pending_resolve(pmix_namespace_t *nptr, pmix_rank_t rank,
-                                               pmix_status_t status, pmix_dmdx_local_t *lcd);
+                                               pmix_status_t status, pmix_scope_t scope,
+                                               pmix_dmdx_local_t *lcd);
 
 PMIX_EXPORT pmix_status_t pmix_server_abort(pmix_peer_t *peer, pmix_buffer_t *buf,
                                             pmix_op_cbfunc_t cbfunc, void *cbdata);
@@ -358,6 +354,14 @@ PMIX_EXPORT pmix_status_t pmix_server_device_dists(pmix_server_caddy_t *cd,
                                                    pmix_buffer_t *buf,
                                                    pmix_device_dist_cbfunc_t cbfunc);
 
+PMIX_EXPORT pmix_status_t pmix_server_refresh_cache(pmix_server_caddy_t *cd,
+                                                    pmix_buffer_t *buf,
+                                                    pmix_op_cbfunc_t cbfunc);
+
+PMIX_EXPORT void pmix_server_query_cbfunc(pmix_status_t status,
+                                          pmix_info_t *info, size_t ninfo, void *cbdata,
+                                          pmix_release_cbfunc_t release_fn, void *release_cbdata);
+
 PMIX_EXPORT extern pmix_server_module_t pmix_host_server;
 PMIX_EXPORT extern pmix_server_globals_t pmix_server_globals;
 
