@@ -15,7 +15,7 @@
  * Copyright (c) 2016-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -118,10 +118,43 @@ PMIX_CLASS_DECLARATION(pmix_cli_result_t);
 #define PMIX_CLI_NUM_CONNECT_RETRIES    "num-connect-retries"       // required
 #define PMIX_CLI_PID                    "pid"                       // required
 #define PMIX_CLI_NAMESPACE              "namespace"                 // required
+#define PMIX_CLI_NSPACE                 "nspace"                    // required
 #define PMIX_CLI_URI                    "uri"                       // required
 #define PMIX_CLI_TIMEOUT                "timeout"                   // required
 #define PMIX_CLI_TMPDIR                 "tmpdir"                    // required
 
+// Allocation request options
+#define PMIX_CLI_REQ_ID                 "request-id"                // required
+#define PMIX_CLI_QUEUE                  "queue"                     // required, short is 'q'
+#define PMIX_CLI_RESOURCES              "resources"                 // required
+#define PMIX_CLI_NODES                  "nodes"                     // required, short is 'N'
+#define PMIX_CLI_IMAGE                  "image"                     // required, short is 'i'
+#define PMIX_CLI_EXCLUDE                "exclude"                   // required, short is 'x'
+#define PMIX_CLI_WAIT_ALL_NODES         "wait-all-nodes"            // none
+#define PMIX_CLI_NODELIST               "nodelist"                  // required, short is 'w'
+#define PMIX_CLI_UID                    "uid"                       // required
+#define PMIX_CLI_GID                    "gid"                       // required
+#define PMIX_CLI_TIME                   "time"                      // required, short is 't'
+#define PMIX_CLI_SIGNAL                 "signal"                    // required
+#define PMIX_CLI_SHARE                  "share"                     // none
+#define PMIX_CLI_EXTEND                 "extend"                    // none
+#define PMIX_CLI_SHRINK                 "shrink"                    // none
+#define PMIX_CLI_NO_SHELL               "no-shell"                  // none
+#define PMIX_CLI_BEGIN                  "begin"                     // required
+#define PMIX_CLI_IMMEDIATE              "immediate"                 // optional, short is 'I'
+#define PMIX_CLI_DEPENDENCY             "dependency"                // required, short is 'd'
+#define PMIX_CLI_DO_NOT_WAIT            "do-not-wait"               // none
+
+// Job control options
+#define PMIX_CLI_PAUSE                  "pause"                     // none
+#define PMIX_CLI_RESUME                 "resume"                    // none
+#define PMIX_CLI_CANCEL                 "cancel"                    // required
+#define PMIX_CLI_KILL                   "kill"                      // none
+#define PMIX_CLI_RESTART                "restart"                   // required
+#define PMIX_CLI_CHKPT                  "checkpoint"                // required
+#define PMIX_CLI_TARGETS                "targets"                   // required
+#define PMIX_CLI_TERMINATE              "terminate"                 // none
+#define PMIX_CLI_PSET_NAME              "pset"                      // required
 
 typedef void (*pmix_cmd_line_store_fn_t)(const char *name, const char *option,
                                          pmix_cli_result_t *results);
@@ -163,7 +196,7 @@ static inline int pmix_cmd_line_get_ninsts(pmix_cli_result_t *results,
     if (NULL == opt) {
         return 0;
     }
-    return pmix_argv_count(opt->values);
+    return PMIx_Argv_count(opt->values);
 }
 
 static inline char* pmix_cmd_line_get_nth_instance(pmix_cli_result_t *results,
@@ -176,7 +209,7 @@ static inline char* pmix_cmd_line_get_nth_instance(pmix_cli_result_t *results,
     if (NULL == opt) {
         return NULL;
     }
-    ninst = pmix_argv_count(opt->values);
+    ninst = PMIx_Argv_count(opt->values);
     if (ninst < idx) {
         return NULL;
     }
@@ -203,11 +236,11 @@ static inline bool pmix_check_cli_option(char *a, char *b)
      * that of the target option */
     if (NULL != strchr(b, '-') ||
         NULL != strchr(a, '-')) {
-        asplit = pmix_argv_split(a, '-');
-        bsplit = pmix_argv_split(b, '-');
-        if (pmix_argv_count(asplit) > pmix_argv_count(bsplit)) {
-            pmix_argv_free(asplit);
-            pmix_argv_free(bsplit);
+        asplit = PMIx_Argv_split(a, '-');
+        bsplit = PMIx_Argv_split(b, '-');
+        if (PMIx_Argv_count(asplit) > PMIx_Argv_count(bsplit)) {
+            PMIx_Argv_free(asplit);
+            PMIx_Argv_free(bsplit);
             return false;
         }
         match = 0;
@@ -218,14 +251,14 @@ static inline bool pmix_check_cli_option(char *a, char *b)
             if (0 == strncasecmp(asplit[n], bsplit[n], len)) {
                 ++match;
             } else {
-                pmix_argv_free(asplit);
-                pmix_argv_free(bsplit);
+                PMIx_Argv_free(asplit);
+                PMIx_Argv_free(bsplit);
                 return false;
             }
         }
-        pmix_argv_free(asplit);
-        pmix_argv_free(bsplit);
-        if (match == pmix_argv_count(asplit)) {
+        PMIx_Argv_free(asplit);
+        PMIx_Argv_free(bsplit);
+        if (match == PMIx_Argv_count(asplit)) {
             /* all provided segments match */
             return true;
         }
@@ -249,8 +282,8 @@ static inline bool pmix_check_cli_option(char *a, char *b)
 
 static inline unsigned int pmix_convert_string_to_time(const char *t)
 {
-    char **tmp = pmix_argv_split(t, ':');
-    int sz = pmix_argv_count(tmp);
+    char **tmp = PMIx_Argv_split(t, ':');
+    int sz = PMIx_Argv_count(tmp);
     unsigned int tm;
 
     /* work upwards from the bottom, where the
@@ -266,7 +299,7 @@ static inline unsigned int pmix_convert_string_to_time(const char *t)
     if (0 <= (sz-4) && NULL != tmp[sz-4]) {
         tm += 24 * 60 * 60 * strtoul(tmp[sz-4], NULL, 10);
     }
-    pmix_argv_free(tmp);
+    PMIx_Argv_free(tmp);
     return tm;
 }
 
@@ -287,7 +320,7 @@ do {                                                                    \
             }                                                           \
         }                                                               \
     }                                                                   \
-    _tail = pmix_argv_join((r)->tail, ' ');                             \
+    _tail = PMIx_Argv_join((r)->tail, ' ');                             \
     pmix_output(0, "TAIL: %s", _tail);                                  \
     free(_tail);                                                        \
     pmix_output(0, "\n");                                               \
