@@ -12,8 +12,8 @@
  * Copyright (c) 2016      Mellanox Technologies, Inc.
  *                         All rights reserved.
  * Copyright (c) 2016      IBM Corporation.  All rights reserved.
- * Copyright (c) 2021-2022 Nanook Consulting  All rights reserved.
- * Copyright (c) 2022      Triad National Security, LLC. All rights reserved.
+ * Copyright (c) 2021-2023 Nanook Consulting.  All rights reserved.
+ * Copyright (c) 2022-2023 Triad National Security, LLC. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -35,6 +35,8 @@
 #include "src/include/pmix_globals.h"
 #include "src/include/pmix_hash_string.h"
 #include "src/mca/bfrops/bfrops.h"
+#include "src/mca/bfrops/base/bfrop_base_tma.h"
+#include "src/mca/gds/base/base.h"
 #include "src/util/pmix_error.h"
 #include "src/util/pmix_output.h"
 
@@ -47,17 +49,22 @@
  */
 typedef struct {
     pmix_object_t super;
-    /* Array of pmix_dstor_t structures containing all data
-     received from this process */
-    pmix_pointer_array_t data;
-    pmix_pointer_array_t quals;
+    /**
+     * Array of pmix_dstor_t structures containing all data received from this
+     * process. Note that these are pointers because the shared-memory TMA
+     * requires that these structures and their data reside on the heap.
+     */
+    pmix_pointer_array_t *data;
+    pmix_pointer_array_t *quals;
 } pmix_proc_data_t;
 static void pdcon(pmix_proc_data_t *p)
 {
-    PMIX_CONSTRUCT(&p->data, pmix_pointer_array_t);
-    pmix_pointer_array_init(&p->data, 128, INT_MAX, 128);
-    PMIX_CONSTRUCT(&p->quals, pmix_pointer_array_t);
-    pmix_pointer_array_init(&p->quals, 1, INT_MAX, 1);
+    pmix_tma_t *const tma = pmix_obj_get_tma(&p->super);
+
+    p->data = PMIX_NEW(pmix_pointer_array_t, tma);
+    pmix_pointer_array_init(p->data, 128, INT_MAX, 128);
+    p->quals = PMIX_NEW(pmix_pointer_array_t, tma);
+    pmix_pointer_array_init(p->quals, 1, INT_MAX, 1);
 }
 static void pddes(pmix_proc_data_t *p)
 {
@@ -66,30 +73,31 @@ static void pddes(pmix_proc_data_t *p)
     pmix_dstor_t *d;
     pmix_qual_t *q;
     pmix_data_array_t *darray;
+    pmix_tma_t *const tma = pmix_obj_get_tma(&p->super);
 
-    for (n=0; n < p->data.size; n++) {
-        d = (pmix_dstor_t*)pmix_pointer_array_get_item(&p->data, n);
+    for (n=0; n < p->data->size; n++) {
+        d = (pmix_dstor_t*)pmix_pointer_array_get_item(p->data, n);
         if (NULL != d) {
-            PMIX_DSTOR_RELEASE(d);
-            pmix_pointer_array_set_item(&p->data, n, NULL);
+            pmix_dstor_release_tma(d, tma);
+            pmix_pointer_array_set_item(p->data, n, NULL);
         }
     }
-    PMIX_DESTRUCT(&p->data);
-    for (n=0; n < p->quals.size; n++) {
-        darray = (pmix_data_array_t*)pmix_pointer_array_get_item(&p->quals, n);
+    PMIX_RELEASE(p->data);
+    for (n=0; n < p->quals->size; n++) {
+        darray = (pmix_data_array_t*)pmix_pointer_array_get_item(p->quals, n);
         if (NULL != darray) {
             q = (pmix_qual_t*)darray->array;
             for (nq=0; nq < darray->size; nq++) {
                 if (NULL != q[nq].value) {
-                    PMIX_VALUE_RELEASE(q[nq].value);
+                    pmix_bfrops_base_tma_value_release(&q[nq].value, tma);
                 }
             }
-            free(darray->array);
-            free(darray);
+            pmix_tma_free(tma, darray->array);
+            pmix_tma_free(tma, darray);
         }
-        pmix_pointer_array_set_item(&p->quals, n, NULL);
+        pmix_pointer_array_set_item(p->quals, n, NULL);
     }
-    PMIX_DESTRUCT(&p->quals);
+    PMIX_RELEASE(p->quals);
 }
 static PMIX_CLASS_INSTANCE(pmix_proc_data_t, pmix_object_t, pdcon, pddes);
 
@@ -112,13 +120,14 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
     pmix_data_array_t *darray;
     pmix_qual_t *qarray;
     size_t n, m = 0;
+    pmix_tma_t *const tma = pmix_obj_get_tma(&table->super);
 
-    pmix_output_verbose(10, pmix_globals.debug_output,
+    pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                         "HASH:STORE:QUAL rank %s key %s",
                         PMIX_RANK_PRINT(rank),
                         (NULL == kin) ? "NULL KVAL" : kin->key);
 
-    if (NULL == kin) {
+    if (PMIX_UNLIKELY(NULL == kin)) {
         return PMIX_ERR_BAD_PARAM;
     }
 
@@ -127,9 +136,9 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
      * refer to the key numerically throughout the internals
      */
     p = pmix_hash_lookup_key(UINT32_MAX, kin->key);
-    if (NULL == p) {
+    if (PMIX_UNLIKELY(NULL == p)) {
         /* we don't know this key */
-        pmix_output_verbose(10, pmix_globals.debug_output,
+        pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                             "%s UNKNOWN KEY: %s",
                             PMIX_NAME_PRINT(&pmix_globals.myid),
                             kin->key);
@@ -139,14 +148,16 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
 
     /* lookup the proc data object for this proc - create
      * it if we don't already have it */
-    if (NULL == (proc_data = lookup_proc(table, rank, true))) {
+    if (PMIX_UNLIKELY(NULL == (proc_data = lookup_proc(table, rank, true)))) {
         return PMIX_ERR_NOMEM;
     }
 
     /* see if we already have this key-value */
     hv = lookup_keyval(proc_data, kid, qualifiers, nquals);
     if (NULL != hv) {
-        if (9 < pmix_output_get_verbosity(pmix_globals.debug_output)) {
+        if (PMIX_UNLIKELY(9 < pmix_output_get_verbosity(pmix_gds_base_framework.framework_output))) {
+            // Note that this doesn't have to use a TMA because it is just a
+            // temporary value.
             char *tmp;
             tmp = PMIx_Value_string(hv->value);
             pmix_output(0, "%s PREEXISTING ENTRY FOR PROC %s KEY %s: %s",
@@ -157,22 +168,24 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
         /* yes we do - so just replace the current value if it changed */
         if (NULL != hv->value) {
             if (PMIX_EQUAL == PMIx_Value_compare(hv->value, kin->value)) {
-                pmix_output_verbose(10, pmix_globals.debug_output,
+                pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                                     "EQUAL VALUE - IGNORING");
                 return PMIX_SUCCESS;
             }
-            if (9 < pmix_output_get_verbosity(pmix_globals.debug_output)) {
+            if (PMIX_UNLIKELY(9 < pmix_output_get_verbosity(pmix_gds_base_framework.framework_output))) {
+                // Note that this doesn't have to use a TMA because it is just a
+                // temporary value.
                 char *tmp;
                 tmp = PMIx_Value_string(kin->value);
                 pmix_output(0, "%s VALUE UPDATING TO: %s",
                             PMIX_NAME_PRINT(&pmix_globals.myid), tmp);
                 free(tmp);
             }
-            PMIX_VALUE_RELEASE(hv->value);
+            pmix_bfrops_base_tma_value_release(&hv->value, tma);
         }
-        /* eventually, we want to eliminate this copy */
-        PMIX_BFROPS_COPY(rc, pmix_globals.mypeer, (void **)&hv->value, kin->value, PMIX_VALUE);
-        if (PMIX_SUCCESS != rc) {
+        /* TODO(skg) eventually, we want to eliminate this copy */
+        rc = pmix_bfrops_base_tma_copy_value(&hv->value, kin->value, PMIX_VALUE, tma);
+        if (PMIX_UNLIKELY(PMIX_SUCCESS != rc)) {
             PMIX_ERROR_LOG(rc);
             return rc;
         }
@@ -180,8 +193,8 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
     }
 
     /* we don't already have it, so create it */
-    PMIX_DSTOR_NEW(hv, kid);
-    if (NULL == hv) {
+    hv = pmix_dstor_new_tma(kid, tma);
+    if (PMIX_UNLIKELY(NULL == hv)) {
         return PMIX_ERR_NOMEM;
     }
     if (NULL != qualifiers) {
@@ -192,30 +205,30 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
             }
         }
         if (0 < m) {
-            darray = (pmix_data_array_t*)pmix_malloc(sizeof(pmix_data_array_t));
-            darray->array = (pmix_qual_t*)pmix_malloc(m * sizeof(pmix_qual_t));
+            darray = (pmix_data_array_t*)pmix_tma_malloc(tma, sizeof(pmix_data_array_t));
+            darray->array = (pmix_qual_t*)pmix_tma_malloc(tma, m * sizeof(pmix_qual_t));
             darray->size = m;
-            hv->qualindex = pmix_pointer_array_add(&proc_data->quals, darray);
+            hv->qualindex = pmix_pointer_array_add(proc_data->quals, darray);
             qarray = (pmix_qual_t*)darray->array;
             for (n=0, m=0; n < nquals; n++) {
                 if (PMIX_INFO_IS_QUALIFIER(&qualifiers[n])) {
                     p = pmix_hash_lookup_key(UINT32_MAX, qualifiers[n].key);
-                    if (NULL == p) {
+                    if (PMIX_UNLIKELY(NULL == p)) {
                         /* we don't know this key */
-                        pmix_output_verbose(10, pmix_globals.debug_output,
+                        pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                                             "%s UNKNOWN KEY: %s",
                                             PMIX_NAME_PRINT(&pmix_globals.myid),
                                             kin->key);
                         erase_qualifiers(proc_data, hv->qualindex);
-                        PMIX_DSTOR_RELEASE(hv);
+                        pmix_dstor_release_tma(hv, tma);
                         return PMIX_ERR_BAD_PARAM;
                     }
                     qarray[n].index = p->index;
-                    PMIX_BFROPS_COPY(rc, pmix_globals.mypeer, (void **)&qarray[m].value, &qualifiers[n].value, PMIX_VALUE);
-                    if (PMIX_SUCCESS != rc) {
+                    rc = pmix_bfrops_base_tma_copy_value(&qarray[m].value, &qualifiers[n].value, PMIX_VALUE, tma);
+                    if (PMIX_UNLIKELY(PMIX_SUCCESS != rc)) {
                         PMIX_ERROR_LOG(rc);
                         erase_qualifiers(proc_data, hv->qualindex);
-                        PMIX_DSTOR_RELEASE(hv);
+                        pmix_dstor_release_tma(hv, tma);
                         return rc;
                     }
                     ++m;
@@ -224,18 +237,20 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
         }
     }
 
-    /* eventually, we want to eliminate this copy */
-    PMIX_BFROPS_COPY(rc, pmix_globals.mypeer, (void **)&hv->value, kin->value, PMIX_VALUE);
-    if (PMIX_SUCCESS != rc) {
+    /* TODO(skg) eventually, we want to eliminate this copy */
+    rc = pmix_bfrops_base_tma_copy_value(&hv->value, kin->value, PMIX_VALUE, tma);
+    if (PMIX_UNLIKELY(PMIX_SUCCESS != rc)) {
         PMIX_ERROR_LOG(rc);
         if (UINT32_MAX != hv->qualindex) {
             /* release the associated qualifiers */
             erase_qualifiers(proc_data, hv->qualindex);
         }
-        PMIX_DSTOR_RELEASE(hv);
+        pmix_dstor_release_tma(hv, tma);
         return rc;
     }
-    if (9 < pmix_output_get_verbosity(pmix_globals.debug_output)) {
+    if (PMIX_UNLIKELY(9 < pmix_output_get_verbosity(pmix_gds_base_framework.framework_output))) {
+        // Note that this doesn't have to use a TMA because it is just a
+        // temporary value.
         char *v = PMIx_Value_string(kin->value);
         pmix_output(0, "%s ADDING KEY %s VALUE %s FOR RANK %s WITH %u QUALS TO TABLE %s",
                     PMIX_NAME_PRINT(&pmix_globals.myid),
@@ -244,10 +259,13 @@ pmix_status_t pmix_hash_store(pmix_hash_table_t *table,
                     table->ht_label);
         free(v);
     }
-    pmix_pointer_array_add(&proc_data->data, hv);
+    pmix_pointer_array_add(proc_data->data, hv);
     return PMIX_SUCCESS;
 }
 
+// TODO(skg) We may have to provide a different fetch entry point with different
+// semantics for gds shmem to further reduce memory usage. For now this seems to
+// work for our current use case.
 pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
                               pmix_rank_t rank,
                               const char *key,
@@ -268,7 +286,7 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
     pmix_data_array_t *darray;
     pmix_qual_t *quals;
 
-    pmix_output_verbose(10, pmix_globals.debug_output,
+    pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                         "%s HASH:FETCH id %s key %s",
                         PMIX_NAME_PRINT(&pmix_globals.myid),
                         PMIX_RANK_PRINT(rank),
@@ -283,7 +301,7 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
         rc = pmix_hash_table_get_first_key_uint32(table, &id, (void **) &proc_data,
                                                   (void **) &node);
         if (PMIX_SUCCESS != rc) {
-            pmix_output_verbose(10, pmix_globals.debug_output,
+            pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                                 "HASH:FETCH[%s:%d] proc data for rank %s not found",
                                 __func__, __LINE__, PMIX_RANK_PRINT(rank));
             return PMIX_ERR_NOT_FOUND;
@@ -310,7 +328,7 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
     while (PMIX_SUCCESS == rc) {
         proc_data = lookup_proc(table, id, false);
         if (NULL == proc_data) {
-            pmix_output_verbose(10, pmix_globals.debug_output,
+            pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                         "HASH:FETCH[%s:%d] proc data for rank %s not found - key %s",
                         __func__, __LINE__,
                         PMIX_RANK_PRINT(rank), key);
@@ -321,14 +339,14 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
          * put by the specified rank */
         if (NULL == key) {
             /* copy the data */
-            for (n=0; n < proc_data->data.size; n++) {
-                hv = (pmix_dstor_t*)pmix_pointer_array_get_item(&proc_data->data, n);
+            for (n=0; n < proc_data->data->size; n++) {
+                hv = (pmix_dstor_t*)pmix_pointer_array_get_item(proc_data->data, n);
                 if (NULL != hv) {
                     p = pmix_hash_lookup_key(hv->index, NULL);
                     if (NULL == p) {
                         return PMIX_ERR_NOT_FOUND;
                     }
-                    pmix_output_verbose(10, pmix_globals.debug_output,
+                    pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                                         "%s FETCH NULL LOOKING AT %s",
                                         PMIX_NAME_PRINT(&pmix_globals.myid), p->name);
                     /* if the rank is UNDEF, we ignore reserved keys */
@@ -337,19 +355,19 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
                         continue;
                     }
                     if (UINT32_MAX != hv->qualindex) {
-                        pmix_output_verbose(10, pmix_globals.debug_output,
+                        pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                                             "%s INCLUDE %s VALUE %u FROM TABLE %s FOR RANK %s",
                                             PMIX_NAME_PRINT(&pmix_globals.myid), p->name,
                                             (unsigned)hv->value->data.size, table->ht_label, PMIX_RANK_PRINT(rank));
                         /* this is a qualified value - need to return it as such */
                         PMIX_KVAL_NEW(kv, PMIX_QUALIFIED_VALUE);
-                        darray = (pmix_data_array_t*)pmix_pointer_array_get_item(&proc_data->quals, hv->qualindex);
+                        darray = (pmix_data_array_t*)pmix_pointer_array_get_item(proc_data->quals, hv->qualindex);
                         quals = (pmix_qual_t*)darray->array;
                         nq = darray->size;
                         PMIX_DATA_ARRAY_CREATE(darray, nq+1, PMIX_INFO);
                         iptr = (pmix_info_t*)darray->array;
                         /* the first location is the actual value */
-                        PMIX_LOAD_KEY(&iptr[0].key, p->string);
+                        PMIX_LOAD_KEY(iptr[0].key, p->string);
                         PMIx_Value_xfer(&iptr[0].value, hv->value);
                         /* now add the qualifiers */
                         for (m=0; m < nq; m++) {
@@ -360,7 +378,7 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
                                 PMIX_DATA_ARRAY_FREE(darray);
                                 return PMIX_ERR_BAD_PARAM;
                             }
-                            PMIX_LOAD_KEY(&iptr[m+1].key, p->string);
+                            PMIX_LOAD_KEY(iptr[m+1].key, p->string);
                             PMIx_Value_xfer(&iptr[m+1].value, quals[m].value);
                             PMIX_INFO_SET_QUALIFIER(&iptr[m+1]);
                         }
@@ -381,16 +399,11 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
             if (NULL != hv) {
                 /* create the copy */
                 PMIX_KVAL_NEW(kv, key);
-                PMIX_BFROPS_COPY(rc, pmix_globals.mypeer, (void **)&kv->value, hv->value, PMIX_VALUE);
-                if (PMIX_SUCCESS != rc) {
-                    PMIX_ERROR_LOG(rc);
-                    PMIX_RELEASE(kv);
-                    return rc;
-                }
+                PMIx_Value_xfer(kv->value, hv->value);
                 pmix_list_append(kvals, &kv->super);
                 break;
             } else if (!fullsearch) {
-                pmix_output_verbose(10, pmix_globals.debug_output,
+                pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                                     "HASH:FETCH data for key %s not found", key);
                 return PMIX_ERR_NOT_FOUND;
             }
@@ -399,7 +412,7 @@ pmix_status_t pmix_hash_fetch(pmix_hash_table_t *table,
         rc = pmix_hash_table_get_next_key_uint32(table, &id, (void **) &proc_data, node,
                                                  (void **) &node);
         if (PMIX_SUCCESS != rc) {
-            pmix_output_verbose(10, pmix_globals.debug_output,
+            pmix_output_verbose(10, pmix_gds_base_framework.framework_output,
                                 "%s:%d HASH:FETCH data for key %s not found",
                                 __func__, __LINE__, key);
             return PMIX_ERR_NOT_FOUND;
@@ -419,10 +432,11 @@ pmix_status_t pmix_hash_remove_data(pmix_hash_table_t *table,
     int n;
     char *node;
     pmix_regattr_input_t *p;
+    pmix_tma_t *const tma = pmix_obj_get_tma(&table->super);
 
     if (NULL != key) {
         p = pmix_hash_lookup_key(UINT32_MAX, key);
-        if (NULL == p) {
+        if (PMIX_UNLIKELY(NULL == p)) {
             /* we don't know this key */
             return PMIX_ERR_BAD_PARAM;
         }
@@ -439,17 +453,17 @@ pmix_status_t pmix_hash_remove_data(pmix_hash_table_t *table,
                 if (NULL == key) {
                     PMIX_RELEASE(proc_data);
                 } else {
-                    for (n=0; n < proc_data->data.size; n++) {
-                        d = (pmix_dstor_t*)pmix_pointer_array_get_item(&proc_data->data, n);
+                    for (n=0; n < proc_data->data->size; n++) {
+                        d = (pmix_dstor_t*)pmix_pointer_array_get_item(proc_data->data, n);
                         if (NULL != d && kid == d->index) {
                             if (NULL != d->value) {
-                                PMIX_VALUE_RELEASE(d->value);
+                                pmix_bfrops_base_tma_value_release(&d->value, tma);
                             }
                             if (UINT32_MAX != d->qualindex) {
                                 erase_qualifiers(proc_data, d->qualindex);
                             }
-                            free(d);
-                            pmix_pointer_array_set_item(&proc_data->data, n, NULL);
+                            pmix_tma_free(tma, d);
+                            pmix_pointer_array_set_item(proc_data->data, n, NULL);
                             break;
                         }
                     }
@@ -469,17 +483,17 @@ pmix_status_t pmix_hash_remove_data(pmix_hash_table_t *table,
 
     /* if key is NULL, remove all data for this proc */
     if (NULL == key) {
-        for (n=0; n < proc_data->data.size; n++) {
-            d = (pmix_dstor_t*)pmix_pointer_array_get_item(&proc_data->data, n);
+        for (n=0; n < proc_data->data->size; n++) {
+            d = (pmix_dstor_t*)pmix_pointer_array_get_item(proc_data->data, n);
             if (NULL != d) {
                 if (NULL != d->value) {
-                    PMIX_VALUE_RELEASE(d->value);
+                    pmix_bfrops_base_tma_value_release(&d->value, tma);
                 }
                 if (UINT32_MAX != d->qualindex) {
                     erase_qualifiers(proc_data, d->qualindex);
                 }
-                free(d);
-                pmix_pointer_array_set_item(&proc_data->data, n, NULL);
+                pmix_tma_free(tma, d);
+                pmix_pointer_array_set_item(proc_data->data, n, NULL);
             }
         }
         /* remove the proc_data object itself from the jtable */
@@ -490,17 +504,17 @@ pmix_status_t pmix_hash_remove_data(pmix_hash_table_t *table,
     }
 
     /* remove this item */
-    for (n=0; n < proc_data->data.size; n++) {
-        d = (pmix_dstor_t*)pmix_pointer_array_get_item(&proc_data->data, n);
+    for (n=0; n < proc_data->data->size; n++) {
+        d = (pmix_dstor_t*)pmix_pointer_array_get_item(proc_data->data, n);
         if (NULL != d && kid == d->index) {
             if (NULL != d->value) {
-                PMIX_VALUE_RELEASE(d->value);
+                pmix_bfrops_base_tma_value_release(&d->value, tma);
             }
             if (UINT32_MAX != d->qualindex) {
                 erase_qualifiers(proc_data, d->qualindex);
             }
-            free(d);
-            pmix_pointer_array_set_item(&proc_data->data, n, NULL);
+            pmix_tma_free(tma, d);
+            pmix_pointer_array_set_item(proc_data->data, n, NULL);
             break;
         }
     }
@@ -533,8 +547,8 @@ static pmix_dstor_t *lookup_keyval(pmix_proc_data_t *proc_data, uint32_t kid,
         }
     }
 
-    for (n=0; n < proc_data->data.size; n++) {
-        d = (pmix_dstor_t*)pmix_pointer_array_get_item(&proc_data->data, n);
+    for (n=0; n < proc_data->data->size; n++) {
+        d = (pmix_dstor_t*)pmix_pointer_array_get_item(proc_data->data, n);
         if (NULL == d) {
             continue;
         }
@@ -543,7 +557,7 @@ static pmix_dstor_t *lookup_keyval(pmix_proc_data_t *proc_data, uint32_t kid,
                 if (UINT32_MAX == d->qualindex) {
                     continue;
                 }
-                darray = (pmix_data_array_t*)pmix_pointer_array_get_item(&proc_data->quals, d->qualindex);
+                darray = (pmix_data_array_t*)pmix_pointer_array_get_item(proc_data->quals, d->qualindex);
                 qarray = (pmix_qual_t*)darray->array;
                 nfound = 0;
                 /* check the qualifiers */
@@ -594,12 +608,13 @@ static pmix_dstor_t *lookup_keyval(pmix_proc_data_t *proc_data, uint32_t kid,
 static pmix_proc_data_t *lookup_proc(pmix_hash_table_t *jtable, uint32_t id, bool create)
 {
     pmix_proc_data_t *proc_data = NULL;
+    pmix_tma_t *const tma = pmix_obj_get_tma(&jtable->super);
 
     pmix_hash_table_get_value_uint32(jtable, id, (void **) &proc_data);
     if (NULL == proc_data && create) {
         /* The proc clearly exists, so create a data structure for it */
-        proc_data = PMIX_NEW(pmix_proc_data_t);
-        if (NULL == proc_data) {
+        proc_data = PMIX_NEW(pmix_proc_data_t, tma);
+        if (PMIX_UNLIKELY(NULL == proc_data)) {
             return NULL;
         }
         pmix_hash_table_set_value_uint32(jtable, id, proc_data);
@@ -631,6 +646,7 @@ void pmix_hash_register_key(uint32_t inid,
     pmix_pointer_array_set_item(&pmix_globals.keyindex, inid, ptr);
 }
 
+// TODO(skg) We may have to add a TMA wrapper for this call.
 pmix_regattr_input_t* pmix_hash_lookup_key(uint32_t inid,
                                            const char *key)
 {
@@ -691,18 +707,19 @@ static void erase_qualifiers(pmix_proc_data_t *proc,
     pmix_data_array_t *darray;
     pmix_qual_t *qarray;
     size_t n;
+    pmix_tma_t *const tma = pmix_obj_get_tma(&proc->super);
 
-    darray = (pmix_data_array_t*)pmix_pointer_array_get_item(&proc->quals, index);
+    darray = (pmix_data_array_t*)pmix_pointer_array_get_item(proc->quals, index);
     if (NULL == darray || NULL == darray->array) {
         return;
     }
     qarray = (pmix_qual_t*)darray->array;
     for (n=0; n < darray->size; n++) {
         if (NULL != qarray[n].value) {
-            PMIX_VALUE_RELEASE(qarray[n].value);
+            pmix_bfrops_base_tma_value_release(&qarray[n].value, tma);
         }
     }
-    free(qarray);
-    free(darray);
-    pmix_pointer_array_set_item(&proc->quals, index, NULL);
+    pmix_tma_free(tma, qarray);
+    pmix_tma_free(tma, darray);
+    pmix_pointer_array_set_item(proc->quals, index, NULL);
 }
