@@ -42,7 +42,6 @@ static uint32_t nprocs;
 static pmix_proc_t myproc;
 static uint32_t getcount = 0;
 static int msgnum = 0;
-static int exitstatus = 0;
 
 #define PMIX_WAIT_FOR_COMPLETION(a) \
     do {                            \
@@ -70,14 +69,12 @@ static void valcbfunc(pmix_status_t status, pmix_value_t *val, void *cbdata)
                 pmix_output(0, "Rank %d[msg=%d]: PMIx_Get_nb Key %s returned wrong type: %d",
                             myproc.rank, msgnum, key, val->type);
                 ++msgnum;
-                exitstatus = 1;
                 goto done;
             }
             if (1234 != val->data.uint64) {
                 pmix_output(0, "Rank %d[msg=%d]: PMIx_Get_nb Key %s returned wrong value: %d",
                             myproc.rank, msgnum, key, (int) val->data.uint64);
                 ++msgnum;
-                exitstatus = 1;
                 goto done;
             }
         } else if (NULL != strstr(key, "remote")) {
@@ -85,20 +82,17 @@ static void valcbfunc(pmix_status_t status, pmix_value_t *val, void *cbdata)
                 pmix_output(0, "Rank %d[msg=%d]: PMIx_Get_nb Key %s returned wrong type: %d",
                             myproc.rank, msgnum, key, val->type);
                 ++msgnum;
-                exitstatus = 1;
                 goto done;
             }
             if (0 != strcmp(val->data.string, "1234")) {
                 pmix_output(0, "Rank %d[msg=%d]: PMIx_Get_nb Key %s returned wrong value: %s",
                             myproc.rank, msgnum, key, val->data.string);
                 ++msgnum;
-                exitstatus = 1;
                 goto done;
             }
         } else {
             pmix_output(0, "Rank %d[msg=%d]: PMIx_Get_nb returned wrong key: %s", myproc.rank, msgnum, key);
             ++msgnum;
-            exitstatus = 1;
             goto done;
         }
         pmix_output(0, "Rank %d[msg=%d]: PMIx_Get_nb Key %s returned correctly", myproc.rank, msgnum, key);
@@ -106,7 +100,6 @@ static void valcbfunc(pmix_status_t status, pmix_value_t *val, void *cbdata)
         pmix_output(0, "Rank %d[msg=%d]: PMIx_Get_nb Key %s failed: %s", myproc.rank, msgnum, key,
                     PMIx_Error_string(status));
         ++msgnum;
-        exitstatus = 1;
     }
 done:
     free(key);
@@ -172,7 +165,6 @@ int main(int argc, char **argv)
     if (PMIX_SUCCESS != (rc = PMIx_Get(&proc, PMIX_JOB_SIZE, NULL, 0, &val))) {
         pmix_output(0, "Rank %d[msg=%d]: PMIx_Get job size failed: %s", myproc.rank, msgnum, PMIx_Error_string(rc));
         ++msgnum;
-        exitstatus = 1;
         goto done;
     }
     nprocs = val->data.uint32;
@@ -187,7 +179,6 @@ int main(int argc, char **argv)
     if (PMIX_SUCCESS != (rc = PMIx_Store_internal(&myproc, tmp, &value))) {
         pmix_output(0, "Rank %d[msg=%d]: PMIx_Store_internal failed: %d", myproc.rank, msgnum, rc);
         ++msgnum;
-        exitstatus = 1;
         goto done;
     }
 
@@ -197,7 +188,6 @@ int main(int argc, char **argv)
     if (PMIX_SUCCESS != (rc = PMIx_Put(PMIX_LOCAL, tmp, &value))) {
         pmix_output(0, "Rank %d[msg=%d]: PMIx_Put internal failed: %d", myproc.rank, msgnum, rc);
         ++msgnum;
-        exitstatus = 1;
         goto done;
     }
 
@@ -207,7 +197,6 @@ int main(int argc, char **argv)
     if (PMIX_SUCCESS != (rc = PMIx_Put(PMIX_GLOBAL, tmp, &value))) {
         pmix_output(0, "Rank %d[msg=%d]: PMIx_Put internal failed: %d", myproc.rank, msgnum, rc);
         ++msgnum;
-        exitstatus = 1;
         goto done;
     }
 
@@ -222,9 +211,8 @@ int main(int argc, char **argv)
     /* commit the data to the server */
     if (PMIX_SUCCESS != (rc = PMIx_Commit())) {
         pmix_output(0, "Rank %d[msg=%d]: PMIx_Commit failed: %d", myproc.rank, msgnum, rc);
-        ++msgnum;
-        exitstatus = 1;
         goto done;
+        ++msgnum;
     }
 
     if (dofence) {
@@ -236,7 +224,6 @@ int main(int argc, char **argv)
         if (PMIX_SUCCESS != (rc = PMIx_Fence_nb(&proc, 1, NULL, 0, opcbfunc, &active))) {
             pmix_output(0, "Rank %d[msg=%d]: PMIx_Fence failed: %d", myproc.rank, msgnum, rc);
             ++msgnum;
-            exitstatus = 1;
             goto done;
         }
     }
@@ -245,23 +232,22 @@ int main(int argc, char **argv)
     if (PMIX_SUCCESS != (rc = PMIx_Get(&proc, PMIX_LOCAL_PEERS, NULL, 0, &val))) {
         pmix_output(0, "Rank %d[msg=%d]: PMIx_Get local peers failed: %s", myproc.rank, msgnum, PMIx_Error_string(rc));
         ++msgnum;
-        exitstatus = 1;
         goto done;
     }
     /* split the returned string to get the rank of each local peer */
-    peers = pmix_argv_split(val->data.string, ',');
+    peers = PMIx_Argv_split(val->data.string, ',');
     PMIX_VALUE_RELEASE(val);
-    nlocal = pmix_argv_count(peers);
+    nlocal = PMIx_Argv_count(peers);
     if (nprocs == nlocal) {
         all_local = true;
     } else {
         all_local = false;
-        locals = (pmix_rank_t *) malloc(pmix_argv_count(peers) * sizeof(pmix_rank_t));
+        locals = (pmix_rank_t *) malloc(PMIx_Argv_count(peers) * sizeof(pmix_rank_t));
         for (n = 0; NULL != peers[n]; n++) {
             locals[n] = strtoul(peers[n], NULL, 10);
         }
     }
-    pmix_argv_free(peers);
+    PMIx_Argv_free(peers);
 
     /* get the committed data - ask for someone who doesn't exist as well */
     num_gets = 0;
@@ -286,7 +272,6 @@ int main(int argc, char **argv)
             if (PMIX_SUCCESS != (rc = PMIx_Get_nb(&proc, tmp, iptr, ninfo, valcbfunc, tmp))) {
                 pmix_output(0, "Rank %d[msg=%d]: PMIx_Get %s failed: %d", myproc.rank, msgnum, tmp, rc);
                 ++msgnum;
-                exitstatus = 1;
                 goto done;
             }
             ++num_gets;
@@ -296,7 +281,6 @@ int main(int argc, char **argv)
             if (PMIX_SUCCESS != (rc = PMIx_Get_nb(&proc, tmp, iptr, ninfo, valcbfunc, tmp))) {
                 pmix_output(0, "Rank %d[msg=%d]: PMIx_Get %s failed: %d", myproc.rank, msgnum, tmp, rc);
                 ++msgnum;
-                exitstatus = 1;
                 goto done;
             }
             ++num_gets;
@@ -321,7 +305,6 @@ int main(int argc, char **argv)
     if (PMIX_SUCCESS != (rc = PMIx_Fence(&proc, 1, NULL, 0))) {
         pmix_output(0, "Rank %d[msg=%d]: PMIx_Fence failed: %d", myproc.rank, msgnum, rc);
         ++msgnum;
-        exitstatus = 1;
         goto done;
     }
 
@@ -335,5 +318,5 @@ done:
         fprintf(stderr, "Rank %d[msg=%d]: PMIx_Finalize successfully completed\n", myproc.rank, msgnum);
     }
     fflush(stderr);
-    return exitstatus;
+    return (0);
 }
